@@ -7,19 +7,20 @@
  *
  * See the COPYRIGHT file, or do a HELP IRCII COPYRIGHT 
  *
- * @(#)$Id: ignore.h,v 1.6 1995/01/02 13:31:36 mrg stable $
+ * @(#)$Id: ignore.h,v 1.9 1995/08/31 03:51:33 scottr Exp $
  */
 
-#ifndef _IGNORE_H_
-#define _IGNORE_H_
+#ifndef __ignore_h_
+#define __ignore_h_
 
 /* declared in ignore.c */
+	int	is_ignored _((char *, int));
+	int	ignore_combo _((int, int));
+	int	double_ignore _((char *, char *, int));
+	void	ignore _((char *, char *, char *));
+
 extern	int	ignore_usernames;
-extern	int	is_ignored();
 extern	char	highlight_char;
-extern	int	ignore_combo();
-extern	int	double_ignore();
-extern	void	ignore _((char *, char *, char *));
 
 /* Type of ignored nicks */
 #define IGNORE_MSGS	0x0001
@@ -39,4 +40,4 @@ extern	void	ignore _((char *, char *, char *));
 #define DONT_IGNORE 2
 #define HIGHLIGHTED -1
 
-#endif /* _IGNORE_H_ */
+#endif /* __ignore_h_ */
