@@ -7,20 +7,12 @@
  *
  * See the COPYRIGHT file, or do a HELP IRCII COPYRIGHT 
  *
- * @(#)$Id: lastlog.h,v 1.5 1994/07/02 02:38:10 mrg stable $
+ * @(#)$Id: lastlog.h,v 1.12 1997/03/26 03:17:05 mrg Exp $
  */
 
 #ifndef __lastlog_h_
 #define __lastlog_h_
 
-typedef struct	lastlog_stru
-{
-	int	level;
-	char	*msg;
-	struct	lastlog_stru	*next;
-	struct	lastlog_stru	*prev;
-}	Lastlog;
-
 #define LOG_NONE	0x000000
 #define LOG_CURRENT	0x000000
 #define LOG_CRAP	0x000001
@@ -39,21 +31,24 @@ typedef struct	lastlog_stru
 #define LOG_USER2	0x002000
 #define LOG_USER3	0x004000
 #define LOG_USER4	0x008000
+#define LOG_BEEP	0x010000
 
 #define LOG_ALL (LOG_CRAP | LOG_PUBLIC | LOG_MSG | LOG_NOTICE | LOG_WALL | \
 		LOG_WALLOP | LOG_NOTES | LOG_OPNOTE | LOG_SNOTE | LOG_ACTION | \
-		LOG_CTCP | LOG_DCC )
+		LOG_CTCP | LOG_DCC | LOG_BEEP)
 
-extern	void	set_lastlog_level();
-extern	int	set_lastlog_msg_level();
-extern	void	set_lastlog_size();
-extern	void	set_notify_level();
-extern	void	lastlog();
-extern	void	add_to_lastlog();
-extern	char	*bits_to_lastlog_level();
-extern	int	real_lastlog_level();
-extern	int	real_notify_level();
-extern	int	parse_lastlog_level();
-extern	int	islogged();
+#define LOG_DEFAULT	LOG_NONE
+
+	void	set_lastlog_level _((char *));
+	int	set_lastlog_msg_level _((int));
+	void	set_lastlog_size _((int));
+	void	set_notify_level _((char *));
+	void	lastlog _((char *, char *, char *));
+	void	add_to_lastlog _((Window *, char *));
+	char	*bits_to_lastlog_level _((int));
+	int	real_lastlog_level _((void));
+	int	real_notify_level _((void));
+	int	parse_lastlog_level _((char *));
+	int	islogged _((Window *));
 
 #endif /* __lastlog_h_ */
