@@ -9,8 +9,8 @@
  */
 
 #ifndef lint
-static	char	rcsid[] = "@(#)$Id: dcc.c,v 1.50 1995/01/21 01:33:45 mrg Exp $";
-#endif
+static	char	rcsid[] = "@(#)$Id: dcc.c,v 1.90 1997/08/31 06:19:50 mrg Exp $";
+#endif /* lint */
 
 #include "irc.h"
 
@@ -37,40 +37,47 @@ static	char	rcsid[] = "@(#)$Id: dcc.c,v 1.50 1995/01/21 01:33:45 mrg Exp $";
 #include "vars.h"
 #include "window.h"
 #include "output.h"
+#include "newio.h"
 
-extern	void	dcc_chat();
-extern	void	dcc_filesend();
-extern	void	dcc_getfile();
-extern	void	dcc_close();
-extern	void	dcc_talk();
-extern	void	dcc_tmsg();
-extern	void	dcc_rename();
-extern	void	dcc_summon();
-extern	void	dcc_send_raw();
-extern	void	process_incoming_chat();
-extern	void	process_outgoing_file();
-extern	void	process_incoming_file();
-extern	void	process_incoming_talk();
-extern	void	process_incoming_raw();
-extern	void	process_incoming_listen();
+static	void	dcc_chat _((char *));
+static	void	dcc_chat_rename _((char *));
+static	void	dcc_filesend _((char *));
+static	void	dcc_getfile _((char *));
+static	void	dcc_close _((char *));
+static	void	dcc_talk _((char *));
+static	void	dcc_tmsg _((char *));
+static	void	dcc_rename _((char *));
+static	void	dcc_summon _((char *));
+static	void	dcc_send_raw _((char *));
+static	void	process_incoming_chat _((DCC_list *));
+static	void	process_outgoing_file _((DCC_list *));
+static	void	process_incoming_file _((DCC_list *));
+static	void	process_incoming_talk _((DCC_list *));
+static	void	process_incoming_raw _((DCC_list *));
+static	void	process_incoming_listen _((DCC_list *));
+
+#ifndef O_BINARY
+#define O_BINARY 0
+#endif /* O_BINARY */
 
 struct
 {
-	char	*name;
-	void	(*function)();
+	char	*name;	/* *MUST* be in ALL CAPITALS */
+	int	uniq; /* minimum length to be a unique command */
+	void	(*function) _((char *));
 }	dcc_commands[] =
 {
-	{ "CHAT",	dcc_chat },
-	{ "LIST",	dcc_list },
-	{ "SEND",	dcc_filesend },
-	{ "GET",	dcc_getfile },
-	{ "CLOSE",	dcc_close },
-	{ "TALK",	dcc_talk },
-	{ "TMSG",	dcc_tmsg },
-	{ "RENAME",	dcc_rename },
-	{ "SUMMON",	dcc_summon },
-	{ "RAW",	dcc_send_raw },
-	{ NULL,		(void (*)()) NULL }
+	{ "CHAT",	2, dcc_chat },
+	{ "LIST",	1, dcc_list },
+	{ "SEND",	2, dcc_filesend },
+	{ "GET",	1, dcc_getfile },
+	{ "CLOSE",	2, dcc_close },
+	{ "TALK",	2, dcc_talk },
+	{ "TMSG",	2, dcc_tmsg },
+	{ "RENAME",	2, dcc_rename },
+	{ "SUMMON",	2, dcc_summon },
+	{ "RAW",	2, dcc_send_raw },
+	{ NULL,		0, (void (*) _((char *))) NULL }
 };
 
 	char	*dcc_types[] =
@@ -95,7 +102,7 @@ struct	deadlist
 }	*deadlist = NULL;
 
 extern	int	in_ctcp_flag;
-extern	char	MyHostName[];
+extern	char	FAR MyHostName[];
 extern	struct	in_addr	MyHostAddr;
 extern int dgets_errno;
 static	off_t	filesize = 0;
@@ -103,6 +110,10 @@ static	off_t	filesize = 0;
 DCC_list	*ClientList = NULL;
 
 static	void	add_to_dcc_buffer _((DCC_list *, char *));
+static	void	dcc_really_erase _((void));
+static	void	dcc_add_deadclient _((DCC_list *));
+static	int	dcc_open _((DCC_list *));
+static	char	*dcc_time _((time_t));
 
 /*
  * dcc_searchlist searches through the dcc_list and finds the client
@@ -170,27 +181,29 @@ dcc_erase(Element)
 		if (*Client == Element)
 		{
 			*Client = Element->next;
-			close(Element->write);
-			close(Element->read);
+			new_close(Element->write);
+			new_close(Element->read);
 			if (Element->file != -1)
-				close(Element->file);
+				new_close(Element->file);
 			new_free(&Element->description);
 			new_free(&Element->user);
+			new_free(&Element->othername);
 			new_free(&Element->buffer);
 			new_free(&Element);
 			return;
 		}
 }
 
-void
+static	void
 dcc_really_erase()
 {
 	struct deadlist *dies;
 
-	for (;(dies = deadlist) != NULL;)
+	while ((dies = deadlist) != NULL)
 	{
 		deadlist = deadlist->next;
 		dcc_erase(dies->it);
+		new_free(&dies);
 	}
 }
 
@@ -206,6 +219,10 @@ set_dcc_bits(rd, wd)
 
 	for (Client = ClientList; Client != NULL; Client = Client->next)
 	{
+#ifdef DCC_CNCT_PEND
+		if (Client->write != -1 && (Client->flags & DCC_CNCT_PEND))
+			FD_SET(Client->write, wd);
+#endif /* DCC_CNCT_PEND */
 		if (Client->read != -1)
 			FD_SET(Client->read, rd);
 	}
@@ -216,8 +233,9 @@ set_dcc_bits(rd, wd)
  * actions are required.
  */
 void
-dcc_check(Readables)
-	fd_set	*Readables;
+dcc_check(rd, wd)
+	fd_set	*rd,
+		*wd;
 {
 	DCC_list	**Client;
 	struct	timeval	timeout;
@@ -227,14 +245,41 @@ dcc_check(Readables)
 	previous_server = from_server;
 	from_server = (-1);
 	timeout.tv_sec = timeout.tv_usec = 0;
-	message_from((char *) 0, LOG_DCC);
 	lastlog_level = set_lastlog_msg_level(LOG_DCC);
 	for (Client = (&ClientList); *Client != NULL && !break_io_processing;)
 	{
-		if ((*Client)->read != -1 && FD_ISSET((*Client)->read,
-				Readables))
+#ifdef NON_BLOCKING_CONNECTS
+		/*
+		 * run all connect-pending sockets.. suggested by deraadt@theos.com
+		 */
+		if ((*Client)->flags & DCC_CNCT_PEND)
 		{
-			switch((*Client)->flags&DCC_TYPES)
+			struct sockaddr_in	remaddr;
+			int	rl = sizeof(remaddr);
+
+			if (getpeername((*Client)->read, (struct sockaddr *) &remaddr, &rl) != -1)
+			{
+				if ((*Client)->flags & DCC_OFFER)
+				{
+					(*Client)->flags &= ~DCC_OFFER;
+					message_from((*Client)->user, LOG_DCC);
+					if (((*Client)->flags & DCC_TYPES) != DCC_RAW)
+						say("DCC %s connection with %s[%s,%d] established",
+							dcc_types[(*Client)->flags&DCC_TYPES], (*Client)->user,
+							inet_ntoa(remaddr.sin_addr), ntohs(remaddr.sin_port));
+					message_from((char *) 0, LOG_CURRENT);
+				}
+				(*Client)->starttime = time(NULL);
+				(*Client)->flags &= ~DCC_CNCT_PEND;
+				set_blocking((*Client)->read);
+				if ((*Client)->read != (*Client)->write)
+					set_blocking((*Client)->write);
+			} /* else we're not connected yet */
+		}
+#endif /* NON_BLOCKING_CONNECTS */
+		if ((*Client)->read != -1 && FD_ISSET((*Client)->read, rd))
+		{
+			switch((*Client)->flags & DCC_TYPES)
 			{
 			case DCC_CHAT:
 				process_incoming_chat(*Client);
@@ -256,6 +301,7 @@ dcc_check(Readables)
 				break;
 			}
 		}
+done:
 		if ((*Client)->flags & DCC_DELETE)
 		{
 			dcc_add_deadclient(*Client);
@@ -264,7 +310,6 @@ dcc_check(Readables)
 		else
 			Client = (&(**Client).next);
 	}
-	message_from((char *) 0, LOG_CRAP);
 	(void) set_lastlog_msg_level(lastlog_level);
 	dcc_really_erase();
 	from_server = previous_server;
@@ -280,13 +325,21 @@ process_dcc(args)
 	char	*command;
 	int	i;
 	int	lastlog_level;
+	int	len;
 
 	if (!(command = next_arg(args, &args)))
 		return;
+	len = strlen(command);
+	upper(command);
 	for (i = 0; dcc_commands[i].name != NULL; i++)
 	{
-		if (!my_stricmp(dcc_commands[i].name, command))
+		if (!strncmp(dcc_commands[i].name, command, len))
 		{
+			if (len < dcc_commands[i].uniq)
+			{
+				say("DCC command not unique: %s", command );
+				return;
+			}
 			message_from((char *) 0, LOG_DCC);
 			lastlog_level = set_lastlog_msg_level(LOG_DCC);
 			dcc_commands[i].function(args);
@@ -304,25 +357,32 @@ dcc_open(Client)
 {
 	char    *user,
 		*Type;
-	struct	sockaddr_in	localaddr,
-		remaddr;
+	struct	sockaddr_in	localaddr;
 	struct	in_addr 	myip;
-	int	sla,
-		rl = sizeof(remaddr);
+	int	sla;
 	int	old_server;
+#ifndef NON_BLOCKING_CONNECTS
+	struct	sockaddr_in	remaddr;
+	int	rl = sizeof(remaddr);
+#endif /* NON_BLOCKING_CONNECTS */
 
 	user = Client->user;
 	old_server = from_server;
 	if (-1 == from_server)
 		from_server = get_window_server(0);
 	myip.s_addr = server_list[from_server].local_addr.s_addr;
+	if (myip.s_addr == 0 || myip.s_addr == htonl(0x7f000001))
+		myip.s_addr = MyHostAddr.s_addr;
 	Type = dcc_types[Client->flags & DCC_TYPES];
 	if (Client->flags & DCC_OFFER)
 	{
-		message_from((char *) 0, LOG_DCC);
+#ifdef DCC_CNCT_PEND
+		Client->flags |= DCC_CNCT_PEND;
+#endif /* DCC_CNCT_PEND */
 		if ((Client->write = connect_by_number(Client->remport,
-			      inet_ntoa(Client->remote))) < 0)
+			      inet_ntoa(Client->remote), 1)) < 0)
 		{
+			message_from(user, LOG_DCC);
 			say("Unable to create connection: %s",
 				errno ? strerror(errno) : "Unknown Host");
 			message_from((char *) 0, LOG_CURRENT);
@@ -331,25 +391,41 @@ dcc_open(Client)
 			return 0;
 		}
 		Client->read = Client->write;
-		Client->flags &= ~DCC_OFFER;
+		Client->bytes_read = Client->bytes_sent = 0L;
 		Client->flags |= DCC_ACTIVE;
-		(void) getpeername(Client->read, (struct sockaddr *) &remaddr,
-			&rl);
+#ifndef NON_BLOCKING_CONNECTS
+		Client->flags &= ~DCC_OFFER;
+		Client->starttime = time(NULL);
+		if (getpeername(Client->read, (struct sockaddr *) &remaddr, &rl) == -1)
+		{
+			message_from(user, LOG_DCC);
+			say("DCC error: getpeername failed: %s", strerror(errno));
+			message_from((char *) 0, LOG_CURRENT);
+			dcc_erase(Client);
+			from_server = old_server;
+			return 0;
+		}
 		if ((Client->flags & DCC_TYPES) != DCC_RAW)
+		{
+			message_from(user, LOG_DCC);
 			say("DCC %s connection with %s[%s,%d] established",
 				Type, user, inet_ntoa(remaddr.sin_addr),
 				ntohs(remaddr.sin_port));
-		message_from((char *) 0, LOG_CURRENT);
-		Client->starttime = time(NULL);
-		Client->bytes_read = Client->bytes_sent = 0L;
+			message_from((char *) 0, LOG_CURRENT);
+		}
+#endif /* NON_BLOCKING_CONNECTS */
 		from_server = old_server;
 		return 1;
 	}
 	else
 	{
+#ifdef DCC_CNCT_PEND
+		Client->flags |= DCC_WAIT|DCC_CNCT_PEND;
+#else
 		Client->flags |= DCC_WAIT;
-		message_from((char *) 0, LOG_DCC);
-		if ((Client->read = connect_by_number(0, empty_string)) < 0)
+#endif /* DCC_CNCT_PEND */
+		message_from(user, LOG_DCC);
+		if ((Client->read = connect_by_number(0, empty_string, 1)) < 0)
 		{
 			say("Unable to create connection: %s",
 				errno ? strerror(errno) : "Unknown Host");
@@ -371,18 +447,24 @@ dcc_open(Client)
 			else
 				nopath = Client->description;
 
+			/*
+			 * XXX
+			 * should make the case below for the filesize into
+			 * generic off_t2str() function, or something.  this
+			 * cast is merely a STOP-GAP measure.
+			 */
 			if (Client->filesize)
 				send_ctcp(ctcp_type[in_ctcp_flag], user, "DCC",
-					 "%s %s %lu %u %d", Type, nopath,
+					 "%s %s %lu %u %ld", Type, nopath,
 					 (u_long) ntohl(myip.s_addr),
 					 (u_short) ntohs(localaddr.sin_port),
-					 Client->filesize);
+					 (long)Client->filesize);
 			else
 				send_ctcp(ctcp_type[in_ctcp_flag], user, "DCC",
 					 "%s %s %lu %u", Type, nopath,
 					 (u_long) ntohl(myip.s_addr),
 					 (u_short) ntohs(localaddr.sin_port));
-			message_from((char *) 0, LOG_DCC);
+			message_from(user, LOG_DCC);
 			say("Sent DCC %s request to %s", Type, user);
 			message_from((char *) 0, LOG_CURRENT);
 		}
@@ -396,7 +478,7 @@ dcc_open(Client)
 	}	
 }
 
-void
+static void
 dcc_chat(args)
 	char	*args;
 {
@@ -419,8 +501,12 @@ dcc_chat(args)
 }
 
 char	*
+#ifdef __STDC__
+dcc_raw_listen(u_short port)
+#else
 dcc_raw_listen(port)
-	int	port;
+	u_short	port;
+#endif /* __STDC__ */
 {
 	DCC_list	*Client;
 	char	PortName[10];
@@ -438,7 +524,7 @@ dcc_raw_listen(port)
 	}
 	sprintf(PortName, "%d", port);
 	Client = dcc_searchlist("raw_listen", PortName, DCC_RAW_LISTEN, 1, (char *) 0);
-	if (Client->flags&DCC_WAIT)
+	if (Client->flags & DCC_ACTIVE)
 	{
 		say("A previous DCC RAW_LISTEN on %s exists", PortName);
 		(void) set_lastlog_msg_level(lastlog_level);
@@ -478,9 +564,13 @@ dcc_raw_listen(port)
 }
 
 char	*
+#ifdef __STDC__
+dcc_raw_connect(char *host, u_short port)
+#else
 dcc_raw_connect(host, port)
 	char	*host;
 	u_short	port;
+#endif /* __STDC__ */
 {
 	DCC_list	*Client;
 	char	PortName[10];
@@ -538,7 +628,7 @@ char    *talk_errors[] =
 						back then? */
 };
 
-void
+static	void
 dcc_talk(args)
 	char	*args;
 {
@@ -585,7 +675,7 @@ dcc_talk(args)
 	else
 		bcopy((char *) &MyHostAddr, (char *) &(Client->remote),
 			sizeof(struct in_addr));
-	if ((Client->file = connect_by_number(-1, empty_string)) < 0)
+	if ((Client->file = connect_by_number(-1, empty_string, 0)) < 0)
 	{
 		say("Unable to create DCC TALK connection: %s",
 				errno ? strerror(errno) : "Unknown Host");
@@ -595,29 +685,29 @@ dcc_talk(args)
 	say("Checking for invitation on caller's machine");
 	if (!(status = send_talk_control(Client, DCC_TALK_CHECK)))
 	{
-		close(Client->file);
+		new_close(Client->file);
 		dcc_erase(Client);
 		say("DCC TALK: connection timed out");
 		return;
 	}
 	if (--status || (Client->read = connect_by_number(Client->remport,
-				inet_ntoa(Client->remote))) < 0)
+				inet_ntoa(Client->remote), 0)) < 0)
 	{
 		say("Inviting %s@%s", Client->user, Client->description);
-		if ((Client->read = connect_by_number(0, empty_string)) == -1 ||
+		if ((Client->read = connect_by_number(0, empty_string, 0)) == -1 ||
 		    !send_talk_control(Client, DCC_TALK_INVITE) ||
 		    !(status=send_talk_control(Client, DCC_TALK_ANNOUNCE)))
 		{
 			send_talk_control(Client, DCC_TALK_DELETE_LOCAL);
-			close(Client->read);
-			close(Client->file);
+			new_close(Client->read);
+			new_close(Client->file);
 			dcc_erase(Client);
 			return;
 		}
 		if (--status)
 		{
-			close(Client->read);
-			close(Client->file);
+			new_close(Client->read);
+			new_close(Client->file);
 			dcc_erase(Client);
 			say("DCC TALK: %s", talk_errors[status]);
 			return;
@@ -636,7 +726,7 @@ dcc_talk(args)
 	}
 }
 
-void
+static	void
 dcc_summon(args)
 	char	*args;
 {
@@ -671,7 +761,7 @@ dcc_summon(args)
 	else
 		bcopy((char *) &MyHostAddr, (char *) &(Client->remote),
 			sizeof(struct in_addr));
-	if ((Client->file = connect_by_number(-1, empty_string)) < 0)
+	if ((Client->file = connect_by_number(-1, empty_string, 0)) < 0)
 	{
 		say("Unable to create DCC SUMMON connection: %s",
 				errno ? strerror(errno) : "Unknown Host");
@@ -680,7 +770,7 @@ dcc_summon(args)
 	if (0 == send_talk_control(Client, DCC_TALK_SUMMON))
 		say("DCC SUMMON: connection timed out");
 	send_talk_control(Client, DCC_TALK_DELETE_SUMMON);
-	close(Client->file);
+	new_close(Client->file);
 	dcc_erase(Client);
 }
 
@@ -768,8 +858,7 @@ send_talk_control(Client, MessageType)
 		case 1:
 			do
 			{
-				recv(Client->file, &Response,
-					sizeof(Response), 0);
+				recv(Client->file, (char *) &Response, sizeof(Response), 0);
 				FD_ZERO(&selset);
 				FD_SET(Client->file, &selset);
 				timeout.tv_sec = 0;
@@ -793,7 +882,7 @@ send_talk_control(Client, MessageType)
 	return 0;
 }
 
-void
+static	void
 dcc_filesend(args)
 	char	*args;
 {
@@ -810,14 +899,14 @@ dcc_filesend(args)
 		say("You are not permitted to use DCC to exchange files");
 		return;
 	}
-#endif
+#endif /* DAEMON_UID */
 	if (0 == (user = next_arg(args, &args)) ||
 	    0 == (filename = next_arg(args, &args)))
 	{
 		say("You must supply a nickname and filename for DCC SEND");
 		return;
 	}
-	if (*filename == '/')
+	if (IS_ABSOLUTE_PATH(filename))
 	{
 		strcpy(FileBuf, filename);
 	}
@@ -843,24 +932,35 @@ dcc_filesend(args)
 		return;
 	}
 	stat_file(FileBuf, &stat_buf);
+/* some unix didn't have this ???? */
+#ifdef S_IFDIR
 	if (stat_buf.st_mode & S_IFDIR)
 	{
 		yell("Cannot send a directory");
 		return;
 	}
+#endif /* S_IFDER */
 	if (scanstr(FileBuf, "/etc/"))
 	{
 		yell("Send request rejected");
 		return;
 	}
-	if (strlen(FileBuf) >= 7 &&
-	    0 == strcmp(FileBuf + strlen(FileBuf) - 7, "/passwd"))
+	if ((int) strlen(FileBuf) >= 7 && 0 == strcmp(FileBuf + strlen(FileBuf) - 7, "/passwd"))
 	{
 		yell("Send request rejected");
 		return;
 	}
 	filesize = stat_buf.st_size;
 	Client = dcc_searchlist(FileBuf, user, DCC_FILEOFFER, 1, filename);
+	if ((Client->file = open(Client->description, O_RDONLY | O_BINARY)) == -1)
+	{
+		say("Unable to open %s: %s\n", Client->description,
+			errno ? strerror(errno) : "Unknown Host");
+		new_close(Client->read);
+		Client->read = Client->write = (-1);
+		Client->flags |= DCC_DELETE;
+		return;
+	}
 	filesize = 0;
 	if ((Client->flags & DCC_ACTIVE) || (Client->flags & DCC_WAIT))
 	{
@@ -872,7 +972,7 @@ dcc_filesend(args)
 }
 
 
-void
+static	void
 dcc_getfile(args)
 	char	*args;
 {
@@ -887,7 +987,7 @@ dcc_getfile(args)
 		say("You are not permitted to use DCC to exchange files");
 		return;
 	}
-#endif
+#endif /* DAEMON_UID */
 	if (0 == (user = next_arg(args, &args)))
 	{
 		say("You must supply a nickname for DCC GET");
@@ -905,7 +1005,10 @@ dcc_getfile(args)
 	}
 	if ((Client->flags & DCC_ACTIVE) || (Client->flags & DCC_WAIT))
 	{
-		say("A previous DCC GET:%s to %s exists", filename, user);
+		if (filename)
+			say("A previous DCC GET:%s to %s exists", filename, user);
+		else
+			say("A previous DCC GET to %s exists", user);
 		return;
 	}
 	if (0 == (Client->flags & DCC_OFFER))
@@ -919,13 +1022,13 @@ dcc_getfile(args)
 	if (!dcc_open(Client))
 		return;
 	if (0 == (fullname = expand_twiddle(Client->description)))
-		malloc_strcpy(fullname, Client->description);
+		malloc_strcpy(&fullname, Client->description);
 	if (-1 == (Client->file = open(fullname,
-				O_WRONLY | O_TRUNC | O_CREAT, 0644)))
+				O_BINARY | O_WRONLY | O_TRUNC | O_CREAT, 0644)))
 	{
 		say("Unable to open %s: %s", Client->description,
 				errno ? strerror(errno) : "<No Error>");
-		close(Client->read);
+		new_close(Client->read);
 		dcc_erase(Client);
 	}
 	new_free(&fullname);
@@ -947,6 +1050,7 @@ register_dcc_offer(user, type, description, address, port, size)
 	u_long	TempLong;
 	unsigned	TempInt;
 	int	do_auto = 0;	/* used in dcc chat collisions */
+	char	*cmd = (char *) 0;
 
 	if (0 != (c = rindex(description, '/')))
 		description = c + 1;
@@ -956,24 +1060,26 @@ register_dcc_offer(user, type, description, address, port, size)
 		filesize = atoi(size);
 	else
 		filesize = 0;
-	if (!my_stricmp(type, "CHAT"))
+	malloc_strcpy(&cmd, type);
+	upper(cmd);
+	if (!strcmp(cmd, "CHAT"))
 		CType = DCC_CHAT;
 #ifndef  DAEMON_UID
-	else if (!strcmp(type, "SEND"))
+	else if (!strcmp(cmd, "SEND"))
 #else
-	else if (!strcmp(type, "SEND") && DAEMON_UID != getuid())
-#endif /*DAEMON_UID*/
+	else if (!strcmp(cmd, "SEND") && DAEMON_UID != getuid())
+#endif /* DAEMON_UID */
 		CType = DCC_FILEREAD;
 	else
 	{
 		say("Unknown DCC %s (%s) received from %s", type, description, user);
-		return;
+		goto out;
 	}
 	Client = dcc_searchlist(description, user, CType, 1, (char *) 0);
 	filesize = 0;
 	if (Client->flags & DCC_WAIT)
 	{
-		close(Client->read);
+		new_close(Client->read);
 		dcc_erase(Client);
 		if (DCC_CHAT == CType)
 		{
@@ -985,13 +1091,13 @@ register_dcc_offer(user, type, description, address, port, size)
 			say("DCC %s collision for %s:%s", type, user,
 				description);
 			send_ctcp_reply(user, "DCC", "DCC %s collision occured while connecting to %s (%s)", type, nickname, description);
-			return;
+			goto out;
 		}
 	}
 	if (Client->flags & DCC_ACTIVE)
 	{
 		say("Received DCC %s request from %s while previous session still active", type, user);
-		return;
+		goto out;
 	}
 	Client->flags |= DCC_OFFER;
 	sscanf(address, "%lu", &TempLong);
@@ -1000,28 +1106,35 @@ register_dcc_offer(user, type, description, address, port, size)
 	Client->remport = TempInt;
 	if (TempInt < 1024)
 	{
-		say("DCC %s (%s) request from %s rejected", type, description, user);
+		say("DCC %s (%s) request from %s rejected [port = %d]", type, description, user, TempInt);
 		dcc_erase(Client);
-		return;
+		goto out;
 	}
 	if ((u_long) 0 == TempLong || 0 == Client->remport)
 	{
 		dcc_erase(Client);
-		return;
+		goto out;
 	}
 	if (do_auto)
 	{
 		say("DCC CHAT already requested by %s, connecting...", user);
 		dcc_chat(user);
 	}
+	/*
+	 * XXX
+	 * should make the case below for the filesize into
+	 * generic off_t2str() function, or something.  this
+	 * cast is merely a STOP-GAP measure.
+	 */
 	else if (Client->filesize)
-		say("DCC %s (%s %d) request received from %s", type, description, Client->filesize, user);
+		say("DCC %s (%s %ld) request received from %s", type, description, (long)Client->filesize, user);
 	else
 		say("DCC %s (%s) request received from %s", type, description, user);
-	return;
+out:
+	new_free(&cmd);
 }
 
-void
+static	void
 process_incoming_chat(Client)
 	DCC_list	*Client;
 {
@@ -1037,10 +1150,10 @@ process_incoming_chat(Client)
 		sra = sizeof(struct sockaddr_in);
 		Client->write = accept(Client->read, (struct sockaddr *)
 			&remaddr, &sra);
-#ifdef  ESIX
+#if defined(ESIX) || defined(_Windows)
 		mark_socket(Client->write);
-#endif
-		close(Client->read);
+#endif /* ESIX */
+		new_close(Client->read);
 		Client->read = Client->write;
 		Client->flags &= ~DCC_WAIT;
 		Client->flags |= DCC_ACTIVE;
@@ -1064,16 +1177,14 @@ process_incoming_chat(Client)
 	switch (bytesread)
 	{
 	case -1:
-#ifdef PHONE
-		say("dcc chat (%s): dgets timed out.  continuing...(%s)", Client->user, tmp);
-#endif
 		add_to_dcc_buffer(Client, tmp);
 		return;
 	case 0:
 		say("DCC CHAT connection to %s lost: %s", Client->user, dgets_errno == -1 ? "Remote end closed connection" : strerror(dgets_errno));
-		close(Client->read);
+		new_close(Client->read);
 		Client->read = Client->write = -1;
 		Client->flags |= DCC_DELETE;
+		/* free_big_buffer(tmp); XXXXXXXXX */
 		return;
 	default:
 		new_free(&Client->buffer);
@@ -1100,7 +1211,7 @@ process_incoming_chat(Client)
 	}
 }
 
-void
+static	void
 process_incoming_listen(Client)
 	DCC_list	*Client;
 {
@@ -1114,7 +1225,7 @@ process_incoming_listen(Client)
 	const char	*Name;
 #else
 	char	*Name;
-#endif
+#endif /* __linux__ || __sgi */
 
 	sra = sizeof(struct sockaddr_in);
 	new_socket = accept(Client->read, (struct sockaddr *) &remaddr,
@@ -1124,9 +1235,9 @@ process_incoming_listen(Client)
 		Name = hp->h_name;
 	else
 		Name = inet_ntoa(remaddr.sin_addr);
-#ifdef  ESIX
+#if defined(ESIX) || defined(_Windows)
 	mark_socket(new_socket);
-#endif
+#endif /* ESIX */
 	sprintf(FdName, "%d", new_socket);
 	NewClient = dcc_searchlist(Name, FdName, DCC_RAW, 1, (char *) 0);
 	NewClient->starttime = time((time_t *) 0);
@@ -1144,7 +1255,7 @@ process_incoming_listen(Client)
 					Client->write);
 }
 
-void
+static	void
 process_incoming_raw(Client)
 	DCC_list	*Client;
 {
@@ -1166,9 +1277,6 @@ process_incoming_raw(Client)
 	switch(bytesread = dgets(bufptr, BIG_BUFFER_SIZE, Client->read, (char *)0))
 	{
 	case -1:
-#ifdef PHONE
-		say("dcc raw (%s): dgets tiomed out.  continuing...(%s)", Client->user, tmp);
-#endif
 		add_to_dcc_buffer(Client, tmp);
 		return;
 	case 0:
@@ -1176,7 +1284,7 @@ process_incoming_raw(Client)
 				Client->user, Client->description))
 			say("DCC RAW connection to %s on %s lost",
 				Client->user, Client->description);
-		close(Client->read);
+		new_close(Client->read);
 		Client->read = Client->write = -1;
 		Client->flags |= DCC_DELETE;
 		(void) dgets_timeout(old_timeout);
@@ -1194,7 +1302,7 @@ process_incoming_raw(Client)
 	}
 }
 
-void
+static	void
 process_incoming_talk(Client)
 	DCC_list	*Client;
 {
@@ -1210,15 +1318,15 @@ process_incoming_talk(Client)
 		sra = sizeof(struct sockaddr_in);
 		Client->write = accept(Client->read, (struct sockaddr *)
 			&remaddr, &sra);
-#ifdef ESIX
+#if defined(ESIX) || defined(_Windows)
 		mark_socket(Client->write);
-#endif
-		close(Client->read);
+#endif /* ESIX */
+		new_close(Client->read);
 		Client->read = Client->write;
 		Client->flags &= ~DCC_WAIT;
 		Client->flags |= DCC_ACTIVE;
 		send_talk_control(Client, DCC_TALK_DELETE_LOCAL);
-		close(Client->file);
+		new_close(Client->file);
 		send(Client->write, "\010\025\027", 3, 0);
 		recv(Client->read, Client->talkchars, 3, 0);
 		Client->bytes_read = Client->bytes_sent = 3;
@@ -1241,14 +1349,11 @@ process_incoming_talk(Client)
 	{
 	case -2: return;
 	case -1:
-#ifdef PHONE
-		say("dcc talk (%s): dgets tiomed out.  continuing...(%s)", Client->user, tmp);
-#endif
 		add_to_dcc_buffer(Client, tmp);
 		return;
 	case 0:
 		say("TALK connection to %s lost", Client->user);
-		close(Client->read);
+		new_close(Client->read);
 		Client->read=Client->write = -1;
 		Client->flags |= DCC_DELETE;
 		(void) dgets_timeout(old_timeout);
@@ -1265,7 +1370,7 @@ process_incoming_talk(Client)
 	}
 }
 
-void
+static	void
 process_outgoing_file(Client)
 	DCC_list	*Client;
 {
@@ -1281,10 +1386,10 @@ process_outgoing_file(Client)
 		sra = sizeof(struct sockaddr_in);
 		Client->write = accept(Client->read,
 				(struct sockaddr *) &remaddr, &sra);
-#ifdef  ESIX
+#if defined(ESIX) || defined(_Windows)
 		mark_socket(Client->write);
-#endif
-		close(Client->read);
+#endif /* ESIX */
+		new_close(Client->read);
 		Client->read = Client->write;
 		Client->flags &= ~DCC_WAIT;
 		Client->flags |= DCC_ACTIVE;
@@ -1292,27 +1397,25 @@ process_outgoing_file(Client)
 		Client->starttime = time(NULL);
 		say("DCC SEND connection to %s[%s,%d] established", Client->user,
 			inet_ntoa(remaddr.sin_addr), ntohs(remaddr.sin_port));
-		if ((Client->file = open(Client->description, O_RDONLY)) == -1)
-		{
-			say("Unable to open %s: %s\n", Client->description,
-				errno ? strerror(errno) : "Unknown Host");
-			close(Client->read);
-			Client->read = Client->write = (-1);
-			Client->flags |= DCC_DELETE;
-			return;
-		}
 	}
 	else 
 	{ 
-		if (recv(Client->read, &bytesrecvd, sizeof(u_32int_t), 0)
-			< sizeof(u_32int_t))
+		if (bytesread = recv(Client->read, (char *) &bytesrecvd, sizeof(u_32int_t), 0) < sizeof(u_32int_t))
 		{
-	       		say("DCC SEND:%s connection to %s lost",
-				Client->description, Client->user);
-			close(Client->read);
+#ifdef _Windows
+			int	recv_error;
+                      recv_error = WSAGetLastError();
+                      if (bytesread == -1 &&
+                          recv_error == WSAEWOULDBLOCK ||
+                          recv_error == WSAEINTR)
+                              return;
+#endif /* _Windows */
+
+	       		say("DCC SEND:%s connection to %s lost: %s", Client->description, Client->user, strerror(errno));
+			new_close(Client->read);
 			Client->read = Client->write = (-1);
 			Client->flags |= DCC_DELETE;
-			close(Client->file);
+			new_close(Client->file);
 			return;
 		}
 		else
@@ -1350,15 +1453,15 @@ process_outgoing_file(Client)
 		sprintf(lame_ultrix, "%2.4g", (sent / (double)xtime));
 		say("DCC SEND:%s to %s completed %s kb/sec",
 			Client->description, Client->user, lame_ultrix);
-		close(Client->read);
+		new_close(Client->read);
 		Client->read = Client->write = -1;
 		Client->flags |= DCC_DELETE;
-		close(Client->file);
+		new_close(Client->file);
 		return;
 	}
 }
 
-void
+static	void
 process_incoming_file(Client)
 	DCC_list	*Client;
 {
@@ -1379,6 +1482,16 @@ process_incoming_file(Client)
 		time_t	xtime = time(NULL) - Client->starttime;
 		double	sent = (double)Client->bytes_read;
 
+#ifdef _Windows
+		{
+			int	recv_error;
+			recv_error = WSAGetLastError();
+			if (bytesread == -1 &&
+			    recv_error == WSAEWOULDBLOCK ||
+			    recv_error == WSAEINTR)
+				return;
+		}
+#endif /* _Windows */
 		if (sent <= 0)
 			sent = 1;
 		sent /= (double)1024.0;
@@ -1387,8 +1500,8 @@ process_incoming_file(Client)
 		sprintf(lame_ultrix, "%2.4g", (sent / (double)xtime));
 		say("DCC GET:%s from %s completed %s kb/sec",
 			Client->description, Client->user, lame_ultrix);
-		close(Client->read);
-		close(Client->file);
+		new_close(Client->read);
+		new_close(Client->file);
 		Client->read = Client->write = (-1);
 		Client->flags |= DCC_DELETE;
 		return;
@@ -1396,7 +1509,7 @@ process_incoming_file(Client)
 	write(Client->file, tmp, bytesread);
 	Client->bytes_read += bytesread;
 	bytestemp = htonl(Client->bytes_read);
-	send(Client->write, &bytestemp, sizeof(u_32int_t), 0);
+	send(Client->write, (char *) &bytestemp, sizeof(u_32int_t), 0);
 }
 
 /* flag == 1 means show it.  flag == 0 used by redirect */
@@ -1437,13 +1550,22 @@ dcc_message_transmit(user, text, type, flag)
 		}
 		break;
 	}
-	if (!(Client = dcc_searchlist(host, user, type, 0, (char *) 0)) ||
-			!(Client->flags&DCC_ACTIVE))
+	if (!(Client = dcc_searchlist(host, user, type, 0, (char *) 0)) || !(Client->flags&DCC_ACTIVE))
 	{
-		say("No active DCC %s:%s connection for %s",
-			dcc_types[type], host?host:"<any>", user);
+		say("No active DCC %s:%s connection for %s", dcc_types[type], host ? host : "<any>", user);
 		return;
 	}
+#ifdef DCC_DCNT_PEND
+	/*
+	 * XXX - should make this buffer
+	 * XXX - just for dcc chat ?  maybe raw dcc too.  hmm.
+	 */
+	if (Client->flags & DCC_CNCT_PEND)
+	{
+		say("DCC %s:%s connection to %s is still connecting...", dcc_types[type], host ? host : "<any>", user);
+		return;
+	}
+#endif /* DCC_DCNT_PEND */
 	lastlog_level = set_lastlog_msg_level(LOG_DCC);
 	message_from(Client->user, LOG_DCC);
 	strmcpy(tmp, text, BIG_BUFFER_SIZE);
@@ -1470,7 +1592,7 @@ dcc_chat_transmit(user,	text)
 	dcc_message_transmit(user, text, DCC_CHAT, 1);
 }
 
-void
+static	void
 dcc_tmsg(args)
 	char	*args;
 {
@@ -1484,7 +1606,7 @@ dcc_tmsg(args)
 	dcc_message_transmit(user, args, DCC_TALK, 1);
 }
 
-void
+static	void
 dcc_send_raw(args)
 	char	*args;
 {
@@ -1531,7 +1653,7 @@ dcc_list(args)
 {
 	DCC_list	*Client;
 	static	char	*format =
-			"%-7.7s %-9.9s %-9.9s %-20.20s %-8.8s %-8.8s %s";
+			"%-7.7s %-9.9s %-10.10s %-20.20s %-8.8s %-8.8s %s";
 	unsigned	flags;
 
 	put_it(format, "Type", "Nick", "Status", "Start time", "Sent", "Read",
@@ -1542,23 +1664,27 @@ dcc_list(args)
 			read[9];
 		char	*time;
 
-		sprintf(sent, "%d", Client->bytes_sent);
-		sprintf(read, "%d", Client->bytes_read);
+		sprintf(sent, "%ld", (long)Client->bytes_sent);
+		sprintf(read, "%ld", (long)Client->bytes_read);
 		time = (Client->starttime) ? dcc_time(Client->starttime) : "",
 		flags = Client->flags;
-		put_it(format, dcc_types[flags&DCC_TYPES],
+		put_it(format, dcc_types[flags & DCC_TYPES],
 				Client->user,
-				flags&DCC_DELETE ? "Closed" :
-				flags&DCC_ACTIVE ? "Active" :
-				flags&DCC_WAIT ? "Waiting" :
-				flags&DCC_OFFER ? "Offered" : "Unknown",
+				flags & DCC_OFFER ? "Offered" :
+				flags & DCC_DELETE ? "Closed" :
+				flags & DCC_ACTIVE ? "Active" :
+				flags & DCC_WAIT ? "Waiting" :
+#ifdef DCC_DCNT_PEND
+				flags & DCC_CNCT_PEND ?	"Connecting" :
+#endif /* DCC_DCNT_PEND */
+				"Unknown",
 				time, sent, read, Client->description);
 		if (*time)
 			new_free(&time);
 	}
 }
 
-void
+static	void
 dcc_close(args)
 	char	*args;
 {
@@ -1568,6 +1694,7 @@ dcc_close(args)
 	char	*user;
 	char	*description;
 	int	CType;
+	char	*cmd = NULL;
 
 	if (!(Type = next_arg(args, &args)) || !(user=next_arg(args, &args)))
 	{
@@ -1575,24 +1702,23 @@ dcc_close(args)
 		return;
 	}
 	description = next_arg(args, &args);
+	malloc_strcpy(&cmd, Type);
+	upper(cmd);
 	for (CType = 0; dcc_types[CType] != NULL; CType++)
-		if (!my_stricmp(Type, dcc_types[CType]))
+		if (!strcmp(cmd, dcc_types[CType]))
 			break;
 	if (!dcc_types[CType])
-	{
 		say("Unknown DCC type: %s", Type);
-		return;
-	}
-	if (Client = dcc_searchlist(description, user, CType, 0, description))
+	else if ((Client = dcc_searchlist(description, user, CType, 0, description)))
 	{
 		flags = Client->flags;
 		if (flags & DCC_DELETE)
 			return;
 		if ((flags & DCC_WAIT) || (flags & DCC_ACTIVE))
 		{
-			close(Client->read);
+			new_close(Client->read);
 			if (Client->file)
-				close(Client->file);
+				new_close(Client->file);
 		}
 		say("DCC %s:%s to %s closed", Type,
 			description ? description : "<any>", user);
@@ -1601,9 +1727,39 @@ dcc_close(args)
 	else
 		say("No DCC %s:%s to %s found", Type,
 			description ? description : "<any>", user);
+	new_free(&cmd);
 }
 
-void
+static void
+dcc_chat_rename(args)
+	char	*args;
+{
+	DCC_list	*Client;
+	char	*user;
+	char	*temp;
+	
+	if (!(user = next_arg(args, &args)) || !(temp = next_arg(args, &args)))
+	{
+		say("you must specify a current DCC CHAT connection, and a new name for it");
+		return;
+	}
+	if (dcc_searchlist("chat", temp, DCC_CHAT, 0, (char *) 0))
+	{
+		say("You already have a DCC CHAT connection with %s, unable to rename.", temp);
+		return;
+	}
+	if ((Client = dcc_searchlist("chat", user, DCC_CHAT, 0, (char *) 0)))
+	{
+		new_free(&(Client->user));
+		malloc_strcpy(&(Client->user), temp);
+		say("DCC CHAT connection with %s renamed to %s", user, temp);
+	}
+	else
+		say("No DCC CHAT connection with %s", user);
+}
+
+
+static	void
 dcc_rename(args)
 	char	*args;
 {
@@ -1613,7 +1769,12 @@ dcc_rename(args)
 	char	*newdesc;
 	char	*temp;
 	
-	if (!(user = next_arg(args, &args)) || !(temp = next_arg(args, &args)))
+	if ((user = next_arg(args, &args)) && my_strnicmp(user, "-chat", strlen(user)) == 0)
+	{
+		dcc_chat_rename(args);
+		return;
+	}
+	if (!user || !(temp = next_arg(args, &args)))
 	{
 		say("you must specify a nick and new filename for DCC RENAME");
 		return;
@@ -1653,7 +1814,7 @@ close_all_dcc()
 {
 	DCC_list *Client;
 
-	for (;(Client = ClientList);)
+	while ((Client = ClientList))
 		dcc_erase(Client);
 }
 
