@@ -9,10 +9,9 @@
  */
 
 #ifndef lint
-static	char	rcsid[] = "@(#)$Id: exec.c,v 1.31 1995/01/23 13:13:10 mrg Exp $";
-#endif
+static	char	rcsid[] = "@(#)$Id: exec.c,v 1.44 1997/04/20 10:35:57 mrg Exp $";
+#endif /* lint */
 
-#define IN_EXEC_C
 #include "irc.h"
 
 #ifdef M_UNIX
@@ -22,7 +21,7 @@ static	char	rcsid[] = "@(#)$Id: exec.c,v 1.31 1995/01/23 13:13:10 mrg Exp $";
 #else /* M_UNIX */
 # ifdef HAVE_SYS_WAIT_H
 #  include <sys/wait.h>
-# endif
+# endif /* HAVE_SYS_WAIT_H */
 #endif /* M_UNIX */
 
 #ifdef ISC
@@ -46,6 +45,8 @@ static	char	rcsid[] = "@(#)$Id: exec.c,v 1.31 1995/01/23 13:13:10 mrg Exp $";
 #include "server.h"
 #include "output.h"
 #include "parse.h"
+#include "dcc.h"
+#include "newio.h"
 
 #if defined(SVR3) && defined(HAVE_SOCKETPAIR)
 /* SVR3's pipe's are *unidirectional*!  We could spend all day pushing
@@ -53,17 +54,30 @@ static	char	rcsid[] = "@(#)$Id: exec.c,v 1.31 1995/01/23 13:13:10 mrg Exp $";
    take the easy way out...like so:
 */
 #define pipe(p) socketpair(AF_UNIX, SOCK_STREAM, 0, (p))
-#endif
+#endif /* SVR3 && HAVE_SOCKETPAIR */
+
+#if !defined(NSIG) && defined(_SIGMAX)
+/* mmm, posix */
+# define NSIG _SIGMAX
+#endif /* !NSIG && _SIGMAX */
 
-static	int	delete_process();
 static	int	wait_index = -1;
 
+static	int	exec_close _((int));
+static	void	start_process _((char *, char *, char *, char *, unsigned int));
+static	void	kill_process _((int, int));
+static	int	delete_process _((int));
+static	void	list_processes _((void));
+static	int	valid_process_index _((int));
+static	void	add_process _((char *, char *, int, int, int, int, char *, char *, unsigned int));
+static	int	is_logical_unique _((char *));
+
 /* Process: the structure that has all the info needed for each process */
 typedef struct
 {
 	char	*name;			/* full process name */
 	char	*logical;
-	int	pid;			/* process-id of process */
+	pid_t	pid;			/* process-id of process */
 	int	p_stdin;		/* stdin description for process */
 	int	p_stdout;		/* stdout descriptor for process */
 	int	p_stderr;		/* stderr descriptor for process */
@@ -84,7 +98,9 @@ static	int	process_list_size = 0;
 
 /*
  * A nice array of the possible signals.  Used by the coredump trapping
- * routines and in the exec.c package 
+ * routines and in the exec.c package .
+ *
+ * We really rely on the signals being all upper case.
  */
 
 #include "sig.inc"
@@ -93,7 +109,7 @@ static	int	process_list_size = 0;
  * exec_close: silly, eh?  Well, it makes the code look nicer.  Or does it
  * really?  No.  But what the hell 
  */
-int
+static	int
 exec_close(des)
 	int	des;
 {
@@ -133,11 +149,11 @@ valid_process_index(proccess)
 
 #ifndef WNOHANG
 # define WNOHANG 1
-#endif
+#endif /* WNOHANG */
 
 #ifndef SIGCLD
 # define SIGCLD SIGCHLD
-#endif
+#endif /* SIGCLD */
 
 volatile static int _child_died = 0;
 
@@ -233,6 +249,7 @@ check_wait_status(wanted)
 			}
 		}
 	}
+	return -1;
 }
 
 /*
@@ -564,9 +581,7 @@ delete_process(process)
 				}
 				if (process_list_size)
 					process_list = (Process **)
-						new_realloc(process_list,
-						sizeof(Process *) *
-						process_list_size);
+						new_realloc((char *) process_list, sizeof(Process *) * process_list_size);
 				else
 				{
 					new_free(&process_list);
@@ -669,8 +684,7 @@ add_process(name, logical, pid, p_stdin, p_stdout, p_stderr, redirect, who, refn
 		}
 	}
 	process_list_size++;
-	process_list = (Process **) new_realloc(process_list, sizeof(Process *)
-		* process_list_size);
+	process_list = (Process **) new_realloc((char *) process_list, sizeof(Process *) * process_list_size);
 	process_list[process_list_size - 1] = (Process *) 0;
 	proc = process_list[i] = (Process *) new_malloc(sizeof(Process));
 	proc->name = (char *) 0;
@@ -708,42 +722,46 @@ kill_process(kill_index, sig)
 {
 	if (process_list && (kill_index < process_list_size) && process_list[kill_index])
 	{
+		pid_t	pgid;
+
 		say("Sending signal %s (%d) to process %d: %s", signals[sig], sig, kill_index, process_list[kill_index]->name);
 #ifdef HAVE_GETPGID
-		kill(-getpgid(process_list[kill_index]->pid), sig);
+		pgid = getpgid(process_list[kill_index]->pid);
 #else
-# ifdef HAVE_GETGID
-		kill(-getgid(process_list[kill_index]->pid, sig);
+# ifdef __sgi
+		pgid = BSDgetpgrp(process_list[kill_index]->pid);
 # else
-#  ifdef BROKEN_GETPGRP
-		kill(-process_list[kill_index]->pid, sig);
+#  ifdef HPUX
+		pgid = getpgrp2(process_list[kill_index]->pid);
 #  else
-#   ifdef __sgi
-		kill(-BSDgetpgrp(process_list[kill_index]->pid), sig);
+#   ifdef mips
+		pgid = getpgrp(process_list[kill_index]->pid);
 #   else
-#    ifdef mips
-		kill(getpgrp(process_list[kill_index]->pid), sig);
+#    ifdef HAVE_GETSID
+		pgid = getsid(process_list[kill_index]->pid);
 #    else
-#     ifdef HPUX
-		killpg(getpgrp2(process_list[kill_index]->pid), sig);
-#     else
-#      if defined(ISC) || defined(MUNIX) || defined(__svr4__) || defined (POSIX)
-		kill(-getpgrp(process_list[kill_index]->pid), sig);
-#      else
-		killpg(getpgrp(process_list[kill_index]->pid), sig);
-#      endif /* defined(ISC) || defined(MUNIX) || defined(__svr4__) */
-#     endif /* HPUX */
-#    endif /* mips */
-#   endif /* __sgi */
-#  endif /* BROKEN_GETPGRP */
-# endif /* HAVE_GETSID */
+#     if defined(ISC) || defined(MUNIX) || defined(BROKEN_GETPGRP) /* || defined(POSIX) */
+		pgid = process_list[kill_index]->pid;		   /* XXX SunOS 4 is mostly */
+#     else							   /* POSIX, but has a real */
+		pgid = getpgrp(process_list[kill_index]->pid);	   /* getpgrp. */
+#     endif /* ISC || MUNIX || BROKEN_GETPGRP */
+#    endif /* HAVE_GETSID */
+#   endif /* mips */
+#  endif /* HPUX */
+# endif /* __sgi */
 #endif /* HAVE_GETPGID */
 
-#ifdef M_UNIX
-		kill((pid_t)process_list[kill_index]->pid, (int)sig) ;
-#else
+#ifndef HAVE_KILLPG
+# define killpg(pg, sig) kill(-(pg), (sig))
+#endif /* HAVE_KILLPG */
+
+		if (pgid == getpid())
+		{
+			yell("--- kill_process got pgid of pid!!!  something is very wrong");
+			return;
+		}
+		killpg(pgid, sig);
 		kill(process_list[kill_index]->pid, sig);
-#endif /* M_UNIX */
 	}
 	else
 		say("There is no process %d", kill_index);
@@ -800,18 +818,18 @@ start_process(name, logical, redirect, who, refnum)
 		say("Unable to start new process: %s", strerror(errno));
 		if (p0[0] != -1)
 		{
-			close(p0[0]);
-			close(p0[1]);
+			new_close(p0[0]);
+			new_close(p0[1]);
 		}
 		if (p1[0] != -1)
 		{
-			close(p1[0]);
-			close(p1[1]);
+			new_close(p1[0]);
+			new_close(p1[1]);
 		}
 		if (p2[0] != -1)
 		{
-			close(p2[0]);
-			close(p2[1]);
+			new_close(p2[0]);
+			new_close(p2[1]);
 		}
 		return;
 	}
@@ -826,16 +844,16 @@ start_process(name, logical, redirect, who, refnum)
 #else
 		setpgrp(0, getpid());
 #endif /* HAVE_SETSID */
-		MY_SIGNAL(SIGINT, SIG_IGN, 0);
+		MY_SIGNAL(SIGINT, (sigfunc *) SIG_IGN, 0);
 		dup2(p0[0], 0);
 		dup2(p1[1], 1);
 		dup2(p2[1], 2);
-		close(p0[0]);
-		close(p0[1]);
-		close(p1[0]);
-		close(p1[1]);
-		close(p2[0]);
-		close(p2[1]);
+		new_close(p0[0]);
+		new_close(p0[1]);
+		new_close(p1[0]);
+		new_close(p1[1]);
+		new_close(p2[0]);
+		new_close(p2[1]);
 		close_all_server();
 		close_all_dcc();
 		close_all_exec();
@@ -863,8 +881,7 @@ start_process(name, logical, redirect, who, refnum)
 				if (cnt == max)
 				{
 					max += 5;
-					args = (char **) new_realloc(args,
-						sizeof(char *) * max);
+					args = (char **) new_realloc((char *) args, sizeof(char *) * max);
 				}
 				args[cnt++] = arg;
 			}
@@ -888,9 +905,9 @@ start_process(name, logical, redirect, who, refnum)
 		_exit(-1);
 		break;
 	default:
-		close(p0[0]);
-		close(p1[1]);
-		close(p2[1]);
+		new_close(p0[0]);
+		new_close(p1[1]);
+		new_close(p2[1]);
 		add_process(name, logical, pid, p0[1], p1[0], p2[0], redirect,
 			who, refnum);
 		break;
@@ -938,6 +955,8 @@ int
 is_process_running(proc_index)
 	int	proc_index;
 {
+	if (proc_index < 0 || proc_index >= process_list_size)
+		return (0);
 	if (process_list && process_list[proc_index])
 		return (!process_list[proc_index]->exited);
 	return (0);
@@ -1015,7 +1034,8 @@ execcmd(command, args, subargs)
 	char	*who = (char *) 0,
 		*logical = (char *) 0,
 		*redirect, /* = (char *) 0, */
-		*flag;
+		*flag,
+		*cmd = (char *) 0;
 	unsigned int	refnum = 0;
 	int	sig,
 		len,
@@ -1050,31 +1070,35 @@ execcmd(command, args, subargs)
 		if (*flag == '-')
 		{
 			len = strlen(++flag);
-			if (my_strnicmp(flag, "OUT", len) == 0)
+			malloc_strcpy(&cmd, flag);
+			upper(cmd);
+			if (strncmp(cmd, "OUT", len) == 0)
 			{
 				redirect = "PRIVMSG";
 				if (!(who = get_channel_by_refnum(0)))
 				{
 					say("No current channel in this window for -OUT");
+					new_free(&cmd);
 					return;
 				}
 			}
-			else if (my_strnicmp(flag, "NAME", len) == 0)
+			else if (strncmp(cmd, "NAME", len) == 0)
 			{
 				logical_flag = 1;
 				if ((logical = next_arg(args, &args)) ==
 						(char *) 0)
 				{
 					say("You must specify a logical name");
+					new_free(&cmd);
 					return;
 				}
 			}
-			else if (my_strnicmp(flag, "WINDOW", len) == 0)
+			else if (strncmp(cmd, "WINDOW", len) == 0)
 			{
 				refnum_flag = 1;
 				refnum = current_refnum();
 			}
-			else if (my_strnicmp(flag, "MSG", len) == 0)
+			else if (strncmp(cmd, "MSG", len) == 0)
 			{
 				if (doing_privmsg)
 					redirect = "NOTICE";
@@ -1084,14 +1108,16 @@ execcmd(command, args, subargs)
 						(char *) 0)
 				{
 					say("No nicknames specified");
+					new_free(&cmd);
 					return;
 				}
 			}
-			else if (my_strnicmp(flag, "CLOSE", len) == 0)
+			else if (strncmp(cmd, "CLOSE", len) == 0)
 			{
 				if ((i = get_process_index(&args)) == -1)
 				{
 					say("Missing process number or logical name.");
+					new_free(&cmd);
 					return;
 				}
 				if (is_process_running(i))
@@ -1103,34 +1129,41 @@ execcmd(command, args, subargs)
 				}
 				else
 					say("No such process running!");
+				new_free(&cmd);
 				return;
 			}
-			else if (my_strnicmp(flag, "NOTICE", len) == 0)
+			else if (strncmp(cmd, "NOTICE", len) == 0)
 			{
 				redirect = "NOTICE";
 				if ((who = next_arg(args, &args)) ==
 						(char *) 0)
 				{
 					say("No nicknames specified");
+					new_free(&cmd);
 					return;
 				}
 			}
-			else if (my_strnicmp(flag, "IN", len) == 0)
+			else if (strncmp(cmd, "IN", len) == 0)
 			{
 				if ((i = get_process_index(&args)) == -1)
 				{
 					say("Missing process number or logical name.");
+					new_free(&cmd);
 					return;
 				}
 				text_to_process(i, args, 1);
+				new_free(&cmd);
 				return;
 			}
 			else
 			{
+				char	*cmd2 = (char *) 0,
+					*p;
+
 				if ((i = get_process_index(&args)) == -1)
 				{
 					say("Invalid process specification");
-					return;
+					goto out;
 				}
 				if ((sig = atoi(flag)) > 0)
 				{
@@ -1138,19 +1171,26 @@ execcmd(command, args, subargs)
 						kill_process(i, sig);
 					else
 						say("Signal number can be from 1 to %d", NSIG);
-					return;
+					goto out;
 				}
+				malloc_strcpy(&cmd2, flag);
+				upper(cmd2);
 				for (sig = 1; sig < NSIG; sig++)
 				{
-					if (!my_strnicmp(signals[sig], flag, len))
+					if (!strncmp(signals[sig], flag, len))
 					{
 						kill_process(i, sig);
-						return;
+						goto out2;
 					}
 				}
 				say("No such signal: %s", flag);
+out2:
+				new_free(&cmd2);
+out:
+				new_free(&cmd);
 				return;
 			}
+			new_free(&cmd);
 		}
 		else
 			break;
