@@ -8,8 +8,8 @@
  */
 
 #ifndef lint
-static	char	rcsid[] = "@(#)$Id: ircsig.c,v 1.3 1994/07/02 02:32:13 mrg stable $";
-#endif
+static	char	rcsid[] = "@(#)$Id: ircsig.c,v 1.6 1996/06/19 07:19:09 mrg Exp $";
+#endif /* lint */
 
 #include "irc.h"
 #include "irc_std.h"
@@ -29,7 +29,11 @@ my_signal(sig_no, sig_handler, misc_flags)
 
         struct sigaction sa, osa;
 
-        sa.sa_handler = sig_handler;
+	/*
+	 * have to (void *) this because of strict type checking
+	 * on some systems -glen
+	 */
+        sa.sa_handler = (void *)sig_handler;
 
         sigemptyset(&sa.sa_mask);
         sigaddset(&sa.sa_mask, sig_no);
@@ -52,8 +56,8 @@ my_signal(sig_no, sig_handler, misc_flags)
 #endif /* SA_RESTART || SA_INTERRUPT */
 
         if (0 > sigaction(sig_no, &sa, &osa))
-                return (SIG_ERR);
+                return ((sigfunc *)SIG_ERR);
 
-        return (osa.sa_handler);
+        return ((sigfunc *)osa.sa_handler);
 }
 #endif /* USE_SIGACTION */
