@@ -1,7 +1,6 @@
 /*
  * keys.c: Does command line parsing, etc 
  *
- *
  * Written By Michael Sandrof
  *
  * Copyright(c) 1990 
@@ -10,8 +9,8 @@
  */
 
 #ifndef lint
-static	char	rcsid[] = "@(#)$Id: keys.c,v 1.14 1995/01/04 13:50:03 mrg stable $";
-#endif
+static	char	rcsid[] = "@(#)$Id: keys.c,v 1.35 1997/06/01 02:43:39 mrg Exp $";
+#endif /* lint */
 
 #include "irc.h"
 
@@ -22,55 +21,25 @@ static	char	rcsid[] = "@(#)$Id: keys.c,v 1.14 1995/01/04 13:50:03 mrg stable $";
 #include "window.h"
 #include "edit.h"
 #include "vars.h"
+#include "translat.h"
+#include "ircterm.h"
+#include "input.h"
+#include "screen.h"
+#include "menu.h"
+#include "ircterm.h"
 
-
-extern	void	input_add_character();
-extern	void	input_backward_word();
-extern	void	input_forward_word();
-extern	void	input_delete_previous_word();
-extern	void	input_delete_next_word();
-extern	void	forward_character();
-extern	void	backward_character();
-extern	void	input_clear_to_bol();
-extern	void	input_clear_line();
-extern	void	input_beginning_of_line();
-extern	void	input_end_of_line();
-extern	void	input_clear_to_eol();
-extern	void	refresh_inputline();
-extern	RETSIGTYPE	refresh_screen();
-extern	void	input_delete_character();
-extern	void	input_backspace();
-extern	void	backward_history();
-extern	void	forward_history();
-extern	void	toggle_insert_mode();
-extern	void	input_transpose_characters();
-extern	void	input_yank_cut_buffer();
-extern	void	send_line();
-extern	void	meta1_char();
-extern	void	meta2_char();
-extern	void	meta3_char();
-extern	void	meta4_char();
-extern	void	irc_quit();
-extern	void	term_pause();
-extern	void	quote_char();
-extern	void	type_text();
-extern	void	parse_text();
-extern	void	toggle_stop_screen();
-extern	void	command_completion();
-extern	void	clear_screen();
-extern	void	enter_digraph();
-extern	void	scrollback_backwards();
-extern	void	scrollback_forwards();
-extern	void	scrollback_end();
-extern	void	scrollback_start();
-extern	void	enter_menu();
+static	int	lookup_function _((char *, int *));
+static	unsigned char	* display_key _((unsigned char));
+static	void	show_binding _((unsigned char, int));
+static	int	parse_key _((unsigned char *));
+static	void	write_binding _((unsigned char, unsigned char, FILE *, int));
 
 /*
  * lookup_function: looks up an irc function by name, and returns the
  * number of functions that match the name, and sets where index points
  * to to be the index of the (first) function found.
  */
-int
+static int
 lookup_function(name, func_index)
 	char	*name;
 	int	*func_index;
@@ -87,7 +56,7 @@ lookup_function(name, func_index)
 		*func_index = -1;
 		for (i = 0; i < NUMBER_OF_FUNCTIONS; i++)
 		{
-			if (my_strnicmp(name, key_names[i].name, len) == 0)
+			if (strncmp(name, key_names[i].name, len) == 0)
 			{
 				cnt++;
 				if (*func_index == -1)
@@ -96,7 +65,7 @@ lookup_function(name, func_index)
 		}
 		if (*func_index == -1)
 			return (0);
-		if (my_stricmp(name, key_names[*func_index].name) == 0)
+		if (strcmp(name, key_names[*func_index].name) == 0)
 			return (1);
 		else
 			return (cnt);
@@ -108,9 +77,13 @@ lookup_function(name, func_index)
  * display_key: converts the character c to a displayable form and returns
  * it.  Very simple indeed 
  */
-unsigned char *
+static	unsigned char	*
+#ifdef __STDC__
+display_key(unsigned char c)
+#else
 display_key(c)
 	unsigned char c;
+#endif /* __STDC__ */
 {
 	static	unsigned char key[3];
 
@@ -138,10 +111,14 @@ display_key(c)
  * meta1 keys, 2 for meta2 keys, anything else for normal keys), this will
  * display the key binding for the key in a nice way
  */
-void
+static void
+#ifdef __STDC__
+show_binding(unsigned char c, int meta)
+#else
 show_binding(c, meta)
 	unsigned char	c;
 	int	meta;
+#endif /* __STDC__ */
 {
 	KeyMap	*map;
 	char	*meta_str;
@@ -185,12 +162,11 @@ show_binding(c, meta)
  * parse_key(), it is converted to "a" and 2 is returned.  Do ya understand
  * this? 
  */
-int
+static int
 parse_key(key_str)
-	char	*key_str;
+	unsigned char	*key_str;
 {
-	char	*ptr1,
-		*ptr2;
+	unsigned char	*ptr1, *ptr2;
 	unsigned char	c;
 	int	meta = 0;
 
@@ -225,54 +201,58 @@ parse_key(key_str)
 			*(ptr2++) = *(ptr1++);
 	}
 	*ptr2 = (char) 0;
-	if (strlen(key_str) > 1)
+	if ((int) strlen((char *) key_str) > 1)
 	{
-		if (my_strnicmp(key_str, "META1-", 6) == 0)
+		char	*cmd = NULL;
+
+		malloc_strcpy(&cmd, (char *)key_str);
+		upper(cmd);
+		if (strncmp(cmd, "META1-", 6) == 0)
 		{
-			strcpy(key_str, key_str + 6);
+			strcpy((char *) key_str, (char *) key_str + 6);
 			meta = 1;
 		}
-		else if (my_strnicmp(key_str, "META2-", 6) == 0)
+		else if (strncmp(cmd, "META2-", 6) == 0)
 		{
-			strcpy(key_str, key_str + 6);
+			strcpy((char *) key_str, (char *) key_str + 6);
 			meta = 2;
 		}
-		else if (my_strnicmp(key_str, "META3-", 6) == 0)
+		else if (strncmp(cmd, "META3-", 6) == 0)
 		{
-			strcpy(key_str, key_str + 6);
+			strcpy((char *) key_str, (char *) key_str + 6);
 			meta = 3;
 		}
-		else if (my_strnicmp(key_str, "META4-", 6) == 0)
+		else if (strncmp(cmd, "META4-", 6) == 0)
 		{
-			strcpy(key_str, key_str + 6);
+			strcpy((char *) key_str, (char *) key_str + 6);
 			meta = 4;
 		}
 		else if (keys[(u_char) *key_str].index == META1_CHARACTER)
 		{
 			meta = 1;
-			strcpy(key_str, key_str + 1);
+			strcpy((char *) key_str, (char *) key_str + 1);
 		}
 		else if (keys[(u_char) *key_str].index == META2_CHARACTER)
 		{
 			meta = 2;
-			strcpy(key_str, key_str + 1);
+			strcpy((char *) key_str, (char *) key_str + 1);
 		}
 		else if (keys[(u_char) *key_str].index == META3_CHARACTER)
 		{
 			meta = 3;
-			strcpy(key_str, key_str + 1);
+			strcpy((char *) key_str, (char *) key_str + 1);
 		}
 		else if (keys[(u_char) *key_str].index == META4_CHARACTER)
 		{
 			meta = 4;
-			strcpy(key_str, key_str + 1);
+			strcpy((char *) key_str, (char *) key_str + 1);
 		}
 		else
 		{
-			say("Illegal key sequence: %s is not a meta-key",
-				display_key(*key_str));
+			say("Illegal key sequence: %s is not a meta-key", display_key(*key_str));
 			return (-1);
 		}
+		new_free(&cmd);
 	}
 	return (meta);
 }
@@ -282,11 +262,15 @@ parse_key(key_str)
  * given meta modifier
  */
 static	void
+#ifdef __STDC__
+bind_it(char *function, char *string, u_char key, int meta)
+#else
 bind_it(function, string, key, meta)
-	u_char	key;
 	char	*function,
 		*string;
+	u_char	key;
 	int	meta;
+#endif /* __STDC__ */
 {
 	KeyMap	*km;
 	int	cnt,
@@ -362,7 +346,7 @@ parsekeycmd(command, args, subargs)
 			say("No such function %s", arg);
 			return;
 		case 1:
-			key_names[i].func();
+			key_names[i].func(0, args);
 			break;
 		default:
 			say("Ambigious function %s", arg);
@@ -388,11 +372,11 @@ bindcmd(command, args, subargs)
 	char	*function;
 	int	meta;
 
-	if ((key = next_arg(args, &args)) != NULL)
+	if ((key = (unsigned char *) next_arg(args, &args)) != NULL)
 	{
 		if ((meta = parse_key(key)) == -1)
 			return;
-		if (strlen(key) > 1)
+		if ((int) strlen((char *) key) > 1)
 		{
 			say("Key sequences may not contain more than two keys");
 			return;
@@ -491,9 +475,10 @@ rbindcmd(command, args, subargs)
 	}
 }
 
-void	(* get_send_line())()
+void (*
+get_send_line _((void))) _((unsigned char, char *))
 {
-	return(key_names[SEND_LINE].func);
+	return (void (*) _((unsigned char, char *))) key_names[SEND_LINE].func;
 }
 
 /*
@@ -504,7 +489,7 @@ void	(* get_send_line())()
  */
 void
 change_send_line(func)
-	void	(*func) ();
+	void	(*func) _((unsigned char, char *));
 {
 	if (func)
 		key_names[SEND_LINE].func = func;
@@ -564,14 +549,15 @@ type(command, args, subargs)
 	}
 }
 
-KeyMapNames key_names[] =
+/* The string values for these *MUST* be in ALL CAPITALS */
+KeyMapNames FAR key_names[] =
 {
 	{ "BACKSPACE",			input_backspace },
 	{ "BACKWARD_CHARACTER",		backward_character },
 	{ "BACKWARD_HISTORY",		backward_history },
 	{ "BACKWARD_WORD",		input_backward_word },
 	{ "BEGINNING_OF_LINE",		input_beginning_of_line },
-	{ "CLEAR_SCREEN",		clear_screen },
+	{ "CLEAR_SCREEN",		irc_clear_screen },
 	{ "COMMAND_COMPLETION",		command_completion },
 	{ "DELETE_CHARACTER",		input_delete_character },
 	{ "DELETE_NEXT_WORD",		input_delete_next_word },
@@ -596,7 +582,7 @@ KeyMapNames key_names[] =
 	{ "QUIT_IRC",			irc_quit },
 	{ "QUOTE_CHARACTER",		quote_char },
 	{ "REFRESH_INPUTLINE",		refresh_inputline },
-	{ "REFRESH_SCREEN",		(void (*) ()) refresh_screen },
+	{ "REFRESH_SCREEN",		refresh_screen },
 	{ "SCROLL_BACKWARD",		scrollback_backwards },
 	{ "SCROLL_END",			scrollback_end },
 	{ "SCROLL_FORWARD",		scrollback_forwards },
@@ -617,7 +603,7 @@ KeyMapNames key_names[] =
 };
 
 
-KeyMap	keys[] =
+KeyMap	FAR keys[] =
 {
 	{ SELF_INSERT,		0, 0,	(char *) 0 },	/* 0 */
 	{ BEGINNING_OF_LINE,	0, 0,	(char *) 0 },
@@ -918,7 +904,7 @@ KeyMap	keys[] =
 };
 
 
-KeyMap	meta1_keys[] =
+KeyMap	FAR meta1_keys[] =
 {
 	{ NOTHING,		0, 0,	(char *) 0 },	/* 0 */
 	{ NOTHING,		0, 0,	(char *) 0 },
@@ -1209,7 +1195,7 @@ KeyMap	meta1_keys[] =
 	{ NOTHING,		0, 0,	(char *) 0 }
 };
 
-KeyMap	meta2_keys[] =
+KeyMap	FAR meta2_keys[] =
 {
 	{ NOTHING,		0, 0,	(char *) 0 },	/* 0 */
 	{ NOTHING,		0, 0,	(char *) 0 },
@@ -1244,7 +1230,7 @@ KeyMap	meta2_keys[] =
 	{ STOP_IRC,		0, 0,	(char *) 0 },
 #else
 	{ NOTHING,		0, 0,	(char *) 0 },
-#endif
+#endif /* ALLOW_STOP_IRC */
 	{ NOTHING,		0, 0,	(char *) 0 },
 	{ NOTHING,		0, 0,	(char *) 0 },
 	{ NOTHING,		0, 0,	(char *) 0 },
@@ -1504,7 +1490,7 @@ KeyMap	meta2_keys[] =
 	{ NOTHING,		0, 0,	(char *) 0 }
 };
 
-KeyMap	meta3_keys[] =
+KeyMap	FAR meta3_keys[] =
 {
 	{ NOTHING,		0, 0,	(char *) 0 },	/* 0 */
 	{ NOTHING,		0, 0,	(char *) 0 },
@@ -1795,7 +1781,7 @@ KeyMap	meta3_keys[] =
 	{ NOTHING,		0, 0,	(char *) 0 }
 };
 
-KeyMap	meta4_keys[] =
+KeyMap	FAR meta4_keys[] =
 {
 	{ NOTHING,		0, 0,	(char *) 0 },	/* 0 */
 	{ NOTHING,		0, 0,	(char *) 0 },
@@ -2093,11 +2079,15 @@ KeyMap	meta4_keys[] =
  * it can be parsed back in later using LOAD or with the -l switch 
  */
 static	void
+#ifdef __STDC__
+write_binding(unsigned char c, unsigned char meta, FILE *fp, int do_all)
+#else
 write_binding(c, meta, fp, do_all)
 	unsigned char	c,
 		meta;
 	FILE	*fp;
 	int	do_all;
+#endif /* __STDC__ */
 {
 	KeyMap	*map;
 	char	*meta_str;
@@ -2153,13 +2143,13 @@ save_bindings(fp, do_all)
 	int	charsize = charset_size();
 
 	for (i = 0; i < charsize; i++)
-		write_binding(i, 0, fp, do_all);
+		write_binding((unsigned char) i, 0, fp, do_all);
 	for (i = 0; i < charsize; i++)
-		write_binding(i, 1, fp, do_all);
+		write_binding((unsigned char) i, 1, fp, do_all);
 	for (i = 0; i < charsize; i++)
-		write_binding(i, 2, fp, do_all);
+		write_binding((unsigned char) i, 2, fp, do_all);
 	for (i = 0; i < charsize; i++)
-		write_binding(i, 3, fp, do_all);
+		write_binding((unsigned char) i, 3, fp, do_all);
 	for (i = 0; i < charsize; i++)
-		write_binding(i, 4, fp, do_all);
+		write_binding((unsigned char) i, 4, fp, do_all);
 }
