@@ -9,8 +9,8 @@
  */
 
 #ifndef lint
-static	char	rcsid[] = "@(#)$Id: notify.c,v 1.15 1995/03/19 03:28:20 mrg Exp $";
-#endif
+static	char	rcsid[] = "@(#)$Id: notify.c,v 1.23 1997/01/17 19:24:31 mrg Exp $";
+#endif /* lint */
 
 #include "irc.h"
 
@@ -33,8 +33,6 @@ typedef	struct	notify_stru
 
 static	NotifyList	*notify_list = (NotifyList *) 0;
 
-extern void	ison_notify();
-
 /* Rewritten, -lynx */
 void
 show_notify_list(all)
@@ -98,7 +96,7 @@ notify(command, args, subargs)
 				nick++;
 				if (*nick)
 				{
-					if ((new = (NotifyList *) remove_from_list(&notify_list, nick)) != NULL)
+					if ((new = (NotifyList *) remove_from_list((List **) &notify_list, nick)) != NULL)
 					{
 						new_free(&(new->nick));
 						new_free(&new);
@@ -109,7 +107,7 @@ notify(command, args, subargs)
 				}
 				else
 				{
-					for (;(new = notify_list);)
+					while ((new = notify_list))
 					{
 						notify_list = new->next;
 						new_free(&new->nick);
@@ -130,7 +128,7 @@ notify(command, args, subargs)
 						say("Wildcards not allowed in NOTIFY nicknames!");
 					else
 					{
-						if ((new = (NotifyList *) remove_from_list(&notify_list, nick)) != NULL)
+						if ((new = (NotifyList *) remove_from_list((List **) &notify_list, nick)) != NULL)
 						{
 							new_free(&(new->nick));
 							new_free(&new);
@@ -139,7 +137,7 @@ notify(command, args, subargs)
 						new->nick = (char *) 0;
 						malloc_strcpy(&(new->nick), nick);
 						new->flag = 0;
-						add_to_list(&notify_list, new);
+						add_to_list((List **) &notify_list, (List *) new);
 						from_server = primary_server;
 						if (get_server_2_6_2(from_server))
 						{
@@ -204,6 +202,13 @@ do_notify()
  * the person is not on the notify list, this call is ignored 
  * doit if passed as 0 means it comes from a join, or a msg, etc, not from
  * an ison reply.  1 is the other..
+ * we also bail out if the primary_server != from_server and the from-server
+ * is valid.
+ *
+ * NOTE:  this function should be called with no particular to_window or other
+ * variables set that would affect what window it would be displayed in.
+ * ideally, a message_from((char *) 0, LOG_CURRENT) should be the what is the
+ * current window level.
  */
 void
 notify_mark(nick, flag, doit)
@@ -214,18 +219,21 @@ notify_mark(nick, flag, doit)
 	NotifyList	*tmp;
 	char	*s = get_string_var(NOTIFY_HANDLER_VAR);
 
-	if (!s || (!doit && 'O' == *s))		/* old notify */
+	if (from_server != primary_server && from_server != -1)
+		return;
+	/* if (!s || (!doit && 'O' == *s)) - this broke notify -gkm *//* old notify */
+	if (!doit && 'O' == *s)		/* old notify */
 		return;	
 	if ('N' == *s)			/* noisy notify */
 		doit = 1;
-	if ((tmp = (NotifyList *) list_lookup(&notify_list, nick,
+	if ((tmp = (NotifyList *) list_lookup((List **) &notify_list, nick,
 			!USE_WILDCARDS, !REMOVE_FROM_LIST)) != NULL)
 	{
 		if (flag)
 		{
 			if (tmp->flag != 1)
 			{
-				if (tmp->flag != -1 && do_hook(NOTIFY_SIGNON_LIST, "%s", nick) && doit)
+				if (tmp->flag != -1 && doit && do_hook(NOTIFY_SIGNON_LIST, "%s", nick))
 					say("Signon by %s detected", nick);
 				/*
 				 * copy the correct case of the nick
@@ -238,7 +246,7 @@ notify_mark(nick, flag, doit)
 		}
 		else
 		{
-			if (tmp->flag == 1 && do_hook(NOTIFY_SIGNOFF_LIST, "%s", nick) && doit)
+			if (tmp->flag == 1 && doit && do_hook(NOTIFY_SIGNOFF_LIST, "%s", nick))
 				say("Signoff by %s detected", nick);
 			tmp->flag = 0;
 		}
