@@ -1,10 +1,15 @@
 #ifndef lint
-static	char	rcsid[] = "@(#)$Id: scandir.c,v 1.25 1995/03/19 03:29:54 mrg Exp $";
-#endif
+static	char	rcsid[] = "@(#)$Id: scandir.c,v 1.34 1997/01/17 16:18:12 mrg Exp $";
+#endif /* lint */
 
-#include "defs.h"
+#include "irc.h"
+#include "ircaux.h"
 
 #ifndef HAVE_SCANDIR
+#include <sys/types.h>
+#include <sys/file.h>
+#include <newio.h>
+
 #if (!defined(ultrix) && !defined(__386BSD__) && !defined(_HPUX_SOURCE)) || defined(HPUX7)
 
 /*
@@ -54,9 +59,7 @@ static char sccsid[] = "@(#)scandir.c	5.3 (Berkeley) 6/18/88";
 **  Scan a directory, collecting all (selected) items into a an array.
 */
 # include <stdio.h>
-# include <sys/types.h>
 # include <dirent.h>
-# include <sys/file.h>
 # ifdef XD88
 #  include <sys/unistd.h>
 # else
@@ -69,21 +72,16 @@ static char sccsid[] = "@(#)scandir.c	5.3 (Berkeley) 6/18/88";
 
 # ifndef DIRSIZ
 #  define DIRSIZ(d) (sizeof(struct dirent) + strlen(d->d_name) + 1) 
-# endif
-
-/* Linked in later. */
-extern char		*new_malloc();
-extern char		*new_realloc();
-extern char		*strcpy();
+# endif /* !DIRSIZ */
 
 int
-scandir(Name, List, Selector, Sorter)
+scandir(Name, dirlist, Selector, Sorter)
 #if defined(__linux__) || defined(__sgi)
     const char            *Name;
 #else
     char		  *Name;
-#endif
-    struct dirent		***List;
+#endif /* __linux__ || __sgi */
+    struct dirent		***dirlist;
     int			 (*Selector)();
     int			 (*Sorter)();
 {
@@ -124,14 +122,14 @@ scandir(Name, List, Selector, Sorter)
 #ifndef __QNX__
 	    names[i - 1]->d_ino = E->d_ino;
 	    names[i - 1]->d_reclen = E->d_reclen;
-#endif
+#endif /* !__QNX__ */
 	    (void) strcpy(names[i - 1]->d_name, E->d_name);
 	}
 
     /* Close things off. */
     names = (struct dirent **) new_realloc((char *) names, (i + 1) * sizeof names[0]);
     names[i] = 0;
-    *List = names;
+    *dirlist = names;
     closedir(Dp);
 
     /* Sort? */
@@ -141,9 +139,7 @@ scandir(Name, List, Selector, Sorter)
     return(i);
 }
 #else
-# include <sys/types.h>
 # include <sys/stat.h>
-# include <sys/file.h>
 # include "irc.h"
 # if defined(ISC22) || defined(ESIX) || defined(HPUX7)
 #  ifdef ESIX
@@ -251,7 +247,7 @@ struct direct **d1,
 {
     return (strcmp((*d1)->d_name, (*d2)->d_name));
 }
-#endif
+#endif /* (!ultrix && !__386BSD__ && !_HPUX_SOURCE) || HPUX7 */
 
 #endif	/* ultrix || __386BSD__ || BSD */
 #endif /* !HAVE_SCANDIR */
