@@ -9,12 +9,12 @@
  */
 
 #ifndef lint
-static	char	rcsid[] = "@(#)$Id: status.c,v 1.19 1995/03/19 03:31:48 mrg Exp $";
-#endif
+static	char	rcsid[] = "@(#)$Id: status.c,v 1.40 1997/03/24 10:20:17 mrg Exp $";
+#endif /* lint */
 
 #include "irc.h"
 
-#include "term.h"
+#include "ircterm.h"
 #include "status.h"
 #include "server.h"
 #include "vars.h"
@@ -29,36 +29,36 @@ static	char	rcsid[] = "@(#)$Id: status.c,v 1.19 1995/03/19 03:31:48 mrg Exp $";
 #include "ircaux.h"
 #include "translat.h"
 
-extern	time_t	time();
-
-extern	char	*update_clock();
-static	char	*convert_format();
-static	char	*status_nickname();
-static	char	*status_query_nick();
-static	char	*status_right_justify();
-static	char	*status_chanop();
-static	char	*status_channel();
-static	char	*status_server();
-static	char	*status_mode();
-static	char	*status_umode();
-static	char	*status_insert_mode();
-static	char	*status_overwrite_mode();
-static	char	*status_away();
-static	char	*status_oper();
-static	char	*status_user0();
-static	char	*status_user1();
-static	char	*status_user2();
-static	char	*status_user3();
-static	char	*status_hold();
-static	char	*status_version();
-static	char	*status_clock();
-static	char	*status_hold_lines();
-static	char	*status_window();
-static	char	*status_mail();
-static	char	*status_refnum();
-static	char	*status_null_function();
-static	char	*status_notify_windows();
+static	char	*convert_format _((char *, int));
+static	char	*status_nickname _((Window *));
+static	char	*status_query_nick _((Window *));
+static	char	*status_right_justify _((Window *));
+static	char	*status_chanop _((Window *));
+static	char	*status_channel _((Window *));
+static	char	*status_server _((Window *));
+static	char	*status_mode _((Window *));
+static	char	*status_umode _((Window *));
+static	char	*status_insert_mode _((Window *));
+static	char	*status_overwrite_mode _((Window *));
+static	char	*status_away _((Window *));
+static	char	*status_oper _((Window *));
+static	char	*status_user0 _((Window *));
+static	char	*status_user1 _((Window *));
+static	char	*status_user2 _((Window *));
+static	char	*status_user3 _((Window *));
+static	char	*status_hold _((Window *));
+static	char	*status_version _((Window *));
+static	char	*status_clock _((Window *));
+static	char	*status_hold_lines _((Window *));
+static	char	*status_window _((Window *));
+static	char	*status_mail _((Window *));
+static	char	*status_refnum _((Window *));
+static	char	*status_null_function _((Window *));
+static	char	*status_notify_windows _((Window *));
+static	char	*status_group _((Window *));
 static	void	status_make_printable _((unsigned char *, int));
+static	void	alarm_switch _((int));
+static	char	*convert_sub_format _((char *, char));
 
 /*
  * Maximum number of "%" expressions in a status line format.  If you change
@@ -69,7 +69,7 @@ static	void	status_make_printable _((unsigned char *, int));
 /* The format statements to build each portion of the status line */
 static	char	*mode_format = (char *) 0;
 static	char	*umode_format = (char *) 0;
-static	char	*status_format = (char *) 0;
+static	char	*status_format[3] = {(char *) 0, (char *) 0, (char *) 0,};
 static	char	*query_format = (char *) 0;
 static	char	*clock_format = (char *) 0;
 static	char	*hold_lines_format = (char *) 0;
@@ -77,15 +77,16 @@ static	char	*channel_format = (char *) 0;
 static	char	*mail_format = (char *) 0;
 static	char	*server_format = (char *) 0;
 static	char	*notify_format = (char *) 0;
+static	char	*group_format = (char *) 0;
 
 /*
  * status_func: The list of status line function in the proper order for
- 3 display.  This list is set in convert_format() 
+ * display.  This list is set in convert_format() 
  */
-static	char	*(*status_func[MAX_FUNCTIONS]) ();
+static	char	*(*status_func[3][MAX_FUNCTIONS]) _((Window *));
 
 /* func_cnt: the number of status line functions assigned */
-static	int	func_cnt;
+static	int	func_cnt[3];
 
 static	int	alarm_hours,		/* hour setting for alarm in 24 hour time */
 		alarm_minutes;		/* minute setting for alarm */
@@ -94,12 +95,11 @@ static	int	alarm_hours,		/* hour setting for alarm in 24 hour time */
 static	struct itimerval clock_timer = { { 10L, 0L }, { 1L, 0L } };
 static	struct itimerval off_timer = { { 0L, 0L }, { 0L, 0L } };
 
-static	RETSIGTYPE alarmed _((int));
+static	RETSIGTYPE alarmed _((void));
 
 /* alarmed: This is called whenever a SIGALRM is received and the alarm is on */
 static	RETSIGTYPE
-alarmed(unused)
-	int	unused;
+alarmed()
 {
 	say("The time is %s", update_clock(GET_TIME));
 	term_beep();
@@ -132,7 +132,7 @@ alarm_switch(flag)
 	else if (alarm_on)
 	{
 		setitimer(ITIMER_REAL, &off_timer, (struct itimerval *) 0);
-		(void) MY_SIGNAL(SIGALRM, SIG_IGN, 0);
+		(void) MY_SIGNAL(SIGALRM, (sigfunc *)SIG_IGN, 0);
 		alarm_on = 0;
 	}
 }
@@ -170,7 +170,7 @@ set_alarm(str)
 		alarm_switch(0);
 		return;
 	}
-
+	
 	c = sscanf(str, " %2[^:]:%2[^paPA]%2s ", hours, minutes, merid);
 	switch (c)
 	{
@@ -188,7 +188,7 @@ set_alarm(str)
 		set_string_var(CLOCK_ALARM_VAR, (char *) 0);
 		return;
 	}
-
+	
 	h = atoi(hours);
 	m = atoi(minutes);
 	if (h >= min_hours && h <= max_hours && isdigit(hours[0]) &&
@@ -335,9 +335,13 @@ reset_clock(unused)
  * mallocs the returned string. 
  */
 static	char	*
+#ifdef __STDC__
+convert_sub_format(char *format, char c)
+#else
 convert_sub_format(format, c)
 	char	*format;
 	char	c;
+#endif /* __STDC__ */
 {
 	char	buffer[BIG_BUFFER_SIZE + 1];
 	static	char	bletch[] = "%% ";
@@ -375,15 +379,15 @@ convert_sub_format(format, c)
 }
 
 static	char	*
-convert_format(str)
-	char	*str;
+convert_format(format, k)
+	char	*format;
+	int	k;
 {
 	char	buffer[BIG_BUFFER_SIZE + 1];
 	char	*ptr,
-		*format,
-		*malloc_ptr = (char *) 0;
-
-	format = get_string_var(STATUS_FORMAT_VAR);
+		*malloc_ptr = (char *) 0; 
+	int	*cp;
+	
 	*buffer = (char) 0;
 	while (format)
 	{
@@ -392,7 +396,8 @@ convert_format(str)
 			*ptr = (char) 0;
 			strmcat(buffer, format, BIG_BUFFER_SIZE);
 			*(ptr++) = '%';
-			if (func_cnt < MAX_FUNCTIONS)
+			cp = &func_cnt[k];
+			if (*cp < MAX_FUNCTIONS)
 			{
 				switch (*(ptr++))
 				{
@@ -401,19 +406,28 @@ convert_format(str)
 					break;
 				case 'N':
 					strmcat(buffer, "%s", BIG_BUFFER_SIZE);
-					status_func[func_cnt++] =
+					status_func[k][(*cp)++] =
 						status_nickname;
 					break;
 				case '>':
 					strmcat(buffer, "%s", BIG_BUFFER_SIZE);
-					status_func[func_cnt++] = status_right_justify;
+					status_func[k][(*cp)++] =
+						status_right_justify;
+					break;
+				case 'G':
+					new_free(&group_format);
+					group_format =
+					convert_sub_format(get_string_var(STATUS_GROUP_VAR), 'G');
+					strmcat(buffer, "%s", BIG_BUFFER_SIZE);
+					status_func[k][(*cp)++] =
+						status_group;
 					break;
 				case 'Q':
 					new_free(&query_format);
 					query_format =
 		convert_sub_format(get_string_var(STATUS_QUERY_VAR), 'Q');
 					strmcat(buffer, "%s", BIG_BUFFER_SIZE);
-					status_func[func_cnt++] =
+					status_func[k][(*cp)++] =
 						status_query_nick;
 					break;
 				case 'F':
@@ -421,112 +435,129 @@ convert_format(str)
 					notify_format = 
 		convert_sub_format(get_string_var(STATUS_NOTIFY_VAR), 'F');
 					strmcat(buffer, "%s", BIG_BUFFER_SIZE);
-					status_func[func_cnt++] =
+					status_func[k][(*cp)++] =
 						status_notify_windows;
 					break;
 				case '@':
 					strmcat(buffer, "%s", BIG_BUFFER_SIZE);
-					status_func[func_cnt++] = status_chanop;
+					status_func[k][(*cp)++] =
+						status_chanop;
 					break;
 				case 'C':
 					new_free(&channel_format);
 					channel_format =
 		convert_sub_format(get_string_var(STATUS_CHANNEL_VAR), 'C');
 					strmcat(buffer, "%s", BIG_BUFFER_SIZE);
-					status_func[func_cnt++]= status_channel;
+					status_func[k][(*cp)++] =
+						status_channel;
 					break;
 				case 'S':
 					new_free(&server_format);
 					server_format =
 		convert_sub_format(get_string_var(STATUS_SERVER_VAR), 'S');
 					strmcat(buffer,"%s",BIG_BUFFER_SIZE);
-					status_func[func_cnt++] = status_server;
+					status_func[k][(*cp)++] =
+						status_server;
 					break;
 				case '+':
 					new_free(&mode_format);
 					mode_format =
 		convert_sub_format(get_string_var(STATUS_MODE_VAR), '+');
 					strmcat(buffer, "%s", BIG_BUFFER_SIZE);
-					status_func[func_cnt++] = status_mode;
+					status_func[k][(*cp)++] =
+						status_mode;
 					break;
 				case '#':
 					new_free(&umode_format);
 					umode_format =
 		convert_sub_format(get_string_var(STATUS_UMODE_VAR), '#');
 					strmcat(buffer, "%s", BIG_BUFFER_SIZE);
-					status_func[func_cnt++] = status_umode;
+					status_func[k][(*cp)++] =
+						status_umode;
 					break;
 				case 'M':
 					new_free(&mail_format);
 					mail_format =
 		convert_sub_format(get_string_var(STATUS_MAIL_VAR), 'M');
 					strmcat(buffer, "%s", BIG_BUFFER_SIZE);
-					status_func[func_cnt++] = status_mail;
+					status_func[k][(*cp)++] =
+						status_mail;
 					break;
 				case 'I':
 					strmcat(buffer, "%s", BIG_BUFFER_SIZE);
-					status_func[func_cnt++] =
+					status_func[k][(*cp)++] =
 						status_insert_mode;
 					break;
 				case 'O':
 					strmcat(buffer, "%s", BIG_BUFFER_SIZE);
-					status_func[func_cnt++] =
+					status_func[k][(*cp)++] =
 						status_overwrite_mode;
 					break;
 				case 'A':
 					strmcat(buffer, "%s", BIG_BUFFER_SIZE);
-					status_func[func_cnt++] = status_away;
+					status_func[k][(*cp)++] =
+						status_away;
 					break;
 				case 'V':
 					strmcat(buffer, "%s", BIG_BUFFER_SIZE);
-					status_func[func_cnt++] = status_version;
+					status_func[k][(*cp)++] =
+						status_version;
 					break;
 				case 'R':
 					strmcat(buffer, "%s", BIG_BUFFER_SIZE);
-					status_func[func_cnt++] = status_refnum;
+					status_func[k][(*cp)++] =
+						status_refnum;
 					break;
 				case 'T':
 					new_free(&clock_format);
 					clock_format =
 		convert_sub_format(get_string_var(STATUS_CLOCK_VAR), 'T');
 					strmcat(buffer, "%s", BIG_BUFFER_SIZE);
-					status_func[func_cnt++] = status_clock;
+					status_func[k][(*cp)++] =
+						status_clock;
 					break;
 				case 'U':
 					strmcat(buffer, "%s", BIG_BUFFER_SIZE);
-					status_func[func_cnt++] = status_user0;
+					status_func[k][(*cp)++] =
+						status_user0;
 					break;
 				case 'H':
 					strmcat(buffer, "%s", BIG_BUFFER_SIZE);
-					status_func[func_cnt++] = status_hold;
+					status_func[k][(*cp)++] =
+						status_hold;
 					break;
 				case 'B':
 					new_free(&hold_lines_format);
 					hold_lines_format =
 		convert_sub_format(get_string_var(STATUS_HOLD_LINES_VAR), 'B');
 					strmcat(buffer, "%s", BIG_BUFFER_SIZE);
-					status_func[func_cnt++] =
+					status_func[k][(*cp)++] =
 						status_hold_lines;
 					break;
 				case '*':
 					strmcat(buffer, "%s", BIG_BUFFER_SIZE);
-					status_func[func_cnt++] = status_oper;
+					status_func[k][(*cp)++] =
+						status_oper;
 					break;
 				case 'W':
 					strmcat(buffer, "%s", BIG_BUFFER_SIZE);
-					status_func[func_cnt++] = status_window;
+					status_func[k][(*cp)++] =
+						status_window;
 					break;
 				case 'X':
 					strmcat(buffer, "%s", BIG_BUFFER_SIZE);
-					status_func[func_cnt++] = status_user1;
+					status_func[k][(*cp)++] =
+						status_user1;
 					break;
 				case 'Y':
 					strmcat(buffer, "%s", BIG_BUFFER_SIZE);
-					status_func[func_cnt++] = status_user2;
+					status_func[k][(*cp)++] =
+						status_user2;
 					break;
 				case 'Z':
 					strmcat(buffer, "%s", BIG_BUFFER_SIZE);
-					status_func[func_cnt++] = status_user3;
+					status_func[k][(*cp)++] =
+						status_user3;
 					break;
 		/* no default..?? - phone, jan 1993 */
 		/* empty is a good default -lynx, mar 93 */
@@ -548,16 +579,31 @@ void
 build_status(format)
 	char	*format;
 {
-	int	i;
+	int	i, k;
 
-	new_free(&status_format);
-	func_cnt = 0;
-	if ((format = get_string_var(STATUS_FORMAT_VAR)) != NULL)
-		status_format = convert_format(format);	/* convert_format
-							 * mallocs for
-							 * us */
-	for (i = func_cnt; i < MAX_FUNCTIONS; i++)
-		status_func[i] = status_null_function;
+	for (k = 0; k < 3; k++) 
+	{
+		new_free(&status_format[k]);
+		func_cnt[k] = 0;
+		switch (k)
+		{
+		case 0 : 
+			format = get_string_var(STATUS_FORMAT_VAR);
+			break;
+			
+		case 1 : 
+			format = get_string_var(STATUS_FORMAT1_VAR);
+			break;
+			
+		case 2 : 
+			format = get_string_var(STATUS_FORMAT2_VAR);
+			break;
+		}
+		if (format != NULL)	/* convert_format mallocs for us */
+			status_format[k] = convert_format(format, k);
+		for (i = func_cnt[k]; i < MAX_FUNCTIONS; i++)
+			status_func[k][i] = status_null_function;
+	}
 	update_all_status();
 }
 
@@ -567,120 +613,154 @@ make_status(window)
 {
 	int	i,
 	len,
+	k,
 	RJustifyPos = -1,
 	RealPosition;
-	char	buffer[BIG_BUFFER_SIZE + 1];
+	Window *affected_window = window;
+	u_char	buffer[BIG_BUFFER_SIZE + 1];
 	char	*func_value[MAX_FUNCTIONS];
 
-	if (!dumb && status_format)
+	for (k = 0 ; k < ((window->double_status) ? 2 : 1); k++)
 	{
-		for (i = 0; i < MAX_FUNCTIONS; i++)
-			func_value[i] = (status_func[i]) (window);
-		buffer[0] = REV_TOG;
-		sprintf(buffer+1, status_format, func_value[0], func_value[1],
-		    func_value[2], func_value[3], func_value[4], func_value[5],
-		    func_value[6], func_value[7], func_value[8], func_value[9],
-		    func_value[10], func_value[11], func_value[12],
-		    func_value[13], func_value[14], func_value[15],
-		    func_value[16], func_value[17], func_value[18],
-		    func_value[19], func_value[20], func_value[21],
-		    func_value[22], func_value[23], func_value[24],
-		    func_value[25], func_value[26], func_value[27],
-		    func_value[28], func_value[29], func_value[30],
-		    func_value[31], func_value[32]);
-		for (i = 0; i < MAX_FUNCTIONS; i++)
-			new_free(&(func_value[i]));
+		int l;
 
-		/*  Patched 26-Mar-93 by Aiken
-		 *  make_window now right-justifies everything after a %>
-		 *  it's also more efficient.
-		 */
-
-		RealPosition = 0;
-		for (i = 0; buffer[i]; i++)
-			/* formfeed is a marker for left/right border */
-			if (buffer[i] == '\f')
+		if (k)
+			l = 2;
+		else if (window->double_status)
+			l = 1;
+		else
+			l = 0;
+			
+		if (!dumb && status_format[l])
+		{
+			for (i = 0; i < MAX_FUNCTIONS; i++)
+				func_value[i] = (status_func[l][i]) (window);
+			buffer[0] = REV_TOG;
+			sprintf((char *) buffer+1, status_format[l],
+				func_value[0], func_value[1], func_value[2],
+				func_value[3], func_value[4], func_value[5],
+				func_value[6], func_value[7], func_value[8],
+				func_value[9], func_value[10], func_value[11],
+				func_value[12], func_value[13], func_value[14],
+				func_value[15], func_value[16], func_value[17],
+				func_value[18], func_value[19], func_value[20],
+				func_value[21], func_value[22], func_value[23],
+				func_value[24], func_value[25], func_value[26],
+				func_value[27], func_value[28], func_value[29],
+				func_value[30], func_value[31],func_value[32]);
+			for (i = 0; i < MAX_FUNCTIONS; i++)
+				new_free(&(func_value[i]));
+			
+			/*  Patched 26-Mar-93 by Aiken
+			 *  make_window now right-justifies everything 
+			 *  after a %>
+			 *  it's also more efficient.
+			 */
+			
+			RealPosition = 0;
+			RJustifyPos = -1;
+			for (i = 0; buffer[i]; i++)
+				/* formfeed is a marker for left/right border*/
+				if (buffer[i] == '\f')
+				{
+					RJustifyPos = i;
+				}
+				else if (buffer[i] != REV_TOG
+					 && buffer[i] != UND_TOG 
+					 && buffer[i] != ALL_OFF 
+					 && buffer[i] != BOLD_TOG)
+				{
+					if (RealPosition == CO)
+					{
+						buffer[i] = '\0';
+						break;
+					}
+					RealPosition++;
+				}
+			
+			/* note that i points to the nul, 
+			   RealPosition is vis.chars */
+			
+			if (RJustifyPos == -1)
 			{
 				RJustifyPos = i;
 			}
-			else if (buffer[i] != REV_TOG && buffer[i] != UND_TOG && buffer[i] != ALL_OFF && buffer[i] != BOLD_TOG)
-			{
-				if (RealPosition == CO)
-				{
-					buffer[i] = '\0';
-					break;
-				}
-				RealPosition++;
-			}
-
-		/* note that i points to the nul, RealPosition is vis.chars */
-
-		if (RJustifyPos == -1)
-		{
-			RJustifyPos = i;
-		}
-		else
-		{
-			/* get rid of the marker */
-			strcpy(&buffer[RJustifyPos], &buffer[RJustifyPos+1]);
-			i--;
-		}
-
-		if (get_int_var(FULL_STATUS_LINE_VAR))
-		{
-			int	diff;
-			char	c;
-
-			if (RJustifyPos == 0)
-				c = ' ';
 			else
-				c = buffer[RJustifyPos - 1];
-
-			diff = CO - RealPosition;
-
-			for ( ; i >= RJustifyPos; i--)
-				buffer[i + diff] = buffer[i];
-
-			for (i++ ; diff > 0 ; diff--, i++)
-				buffer[i] = c;
-		}
-
-		len = strlen(buffer);
-		buffer[len] = ALL_OFF;
-		buffer[len+1] = (char) 0;
-
-		status_make_printable(buffer, len);
-
-	/*
-	 * Thanks to Max Bell (mbell@cie.uoregon.edu) for info about TVI
-	 * terminals and the sg terminal capability 
-	 */
-		RealPosition = 0;
-		if (window->status_line && (SG == -1))
-		{
-			for (i = 0; buffer[i] && window->status_line[i]; i++)
 			{
-				if (buffer[i] != window->status_line[i])
-					break;
-				if (buffer[i] != REV_TOG && buffer[i] != UND_TOG && buffer[i] != ALL_OFF && buffer[i] != BOLD_TOG)
-					RealPosition++;
+				/* get rid of the marker */
+				strcpy((char *) &buffer[RJustifyPos], 
+					(char *) &buffer[RJustifyPos+1]);
+				i--;
+			}
+			
+			if (get_int_var(FULL_STATUS_LINE_VAR))
+			{
+				int	diff;
+				u_char	c;
+				
+				if (RJustifyPos == 0)
+					c = ' ';
+				else
+					c = buffer[RJustifyPos - 1];
+				
+				diff = CO - RealPosition;
+				
+				for ( ; i >= RJustifyPos; i--)
+					buffer[i + diff] = buffer[i];
+				
+				for (i++ ; diff > 0 ; diff--, i++)
+					buffer[i] = c;
+			}
+			
+			len = strlen((char *) buffer);
+			buffer[len] = ALL_OFF;
+			buffer[len+1] =  '\0';
+			
+			status_make_printable(buffer, len);
+			
+	      /*
+	       * Thanks to Max Bell (mbell@cie.uoregon.edu) for info about TVI
+	       * terminals and the sg terminal capability 
+	       */
+			RealPosition = 0;
+			if (window->status_line[k] && (SG == -1))
+			{
+				for (i = 0; buffer[i] && window->status_line[k][i]; i++)
+				{
+					if ((char) buffer[i] != window->status_line[k][i])
+						break;
+					if (buffer[i] != REV_TOG 
+					    && buffer[i] != UND_TOG
+					    && buffer[i] != ALL_OFF
+					    && buffer[i] != BOLD_TOG)
+						RealPosition++;
+				}
+			}
+			else
+				i = 0;
+			
+			if ((len = strlen((char *) buffer + i)) 
+			     || buffer[i] || window->status_line[k] 
+			     || window->status_line[k][i])
+			{
+				Screen *old_current_screen;
+				
+				old_current_screen = current_screen;
+				set_current_screen(window->screen);
+				term_move_cursor(RealPosition, window->bottom
+						 + k);
+/*
+				term_move_cursor(RealPosition, window->bottom
+						 + k - window->double_status);
+*/
+				output_line(buffer, NULL, i);
+				cursor_in_display();
+				if (term_clear_to_eol())
+					term_space_erase(len);
+				malloc_strcpy(&window->status_line[k],
+					      (char *) buffer);
+				set_current_screen(old_current_screen);
 			}
-		}
-		else
-			i = 0;
-		if ((len = strlen(buffer + i)) || buffer[i] || !window->status_line || window->status_line[i])
-		{
-			Screen *old_current_screen;
-
-			old_current_screen = current_screen;
-			set_current_screen(window->screen);
-			term_move_cursor(RealPosition, window->bottom);
-			output_line(buffer, NULL, i);
-			cursor_in_display();
-			if (term_clear_to_eol())
-				term_space_erase(len);
-			malloc_strcpy(&window->status_line, buffer);
-			set_current_screen(old_current_screen);
 		}
 	}
 	cursor_to_input();
@@ -716,16 +796,15 @@ status_server(window)
 			if (server_format)
 			{
 				name = get_server_name(window->server);
-				if ((rest = (char *) index(name, '.')) != NULL)
+				rest = (char *) index(name, '.');
+				if (rest != NULL && my_strnicmp(name, "irc", 3) != 0)
 				{
 					if (is_number(name))
-						sprintf(buffer, server_format,
-							name);
+						sprintf(buffer, server_format, name);
 					else
 					{
 						*rest = '\0';
-						sprintf(buffer, server_format,
-							name);
+						sprintf(buffer, server_format, name);
 						*rest = '.';
 					}
 				}
@@ -736,7 +815,7 @@ status_server(window)
 				*buffer = '\0';
 		}
 		else
-			strcpy(buffer, "No Server");
+			strcpy(buffer, " No Server");
 	}
 	else
 		*buffer = '\0';
@@ -744,6 +823,22 @@ status_server(window)
 	return (ptr);
 }
 
+static	char	*
+status_group(window)
+	Window	*window;
+{
+	char	*ptr = (char *) 0;
+
+	if (window->server_group && group_format)
+	{
+		sprintf(buffer, group_format, find_server_group_name(window->server_group));
+		malloc_strcpy(&ptr, buffer);
+	}
+	else
+		malloc_strcpy(&ptr, empty_string);
+	return (ptr);
+}
+
 static	char	*
 status_query_nick(window)
 	Window	*window;
@@ -833,8 +928,9 @@ status_mode(window)
 {
 	char	*ptr = (char *) 0,
 		*mode;
+	ChannelList	*tmp;
 
-	if (window->current_channel)
+	if (window->current_channel && chan_is_connected(window->current_channel, window->server))
 	{
 		mode = get_channel_mode(window->current_channel,window->server);
 		if (mode && *mode && mode_format)
@@ -857,7 +953,9 @@ status_umode(window)
 	char	localbuf[10];
 	char	*c;
 
-	if ((connected_to_server == 1) && !get_int_var(SHOW_STATUS_ALL_VAR)
+	if (connected_to_server == 0)
+		malloc_strcpy(&ptr, empty_string);
+	else if ((connected_to_server == 1) && !get_int_var(SHOW_STATUS_ALL_VAR)
 	    && (window->screen->current_window != window))
 		malloc_strcpy(&ptr, empty_string);
 	else
@@ -872,7 +970,7 @@ status_umode(window)
 		if (get_server_flag(window->server, USER_MODE_W))
 			*c++ = 'w';
 		*c++ = '\0';
-		if (*localbuf!='\0')
+		if (*localbuf != '\0' && umode_format)
 		{
 			sprintf(buffer, umode_format, localbuf);
 			malloc_strcpy(&ptr, buffer);
@@ -891,6 +989,7 @@ status_chanop(window)
 		*text;
 
 	if (window->current_channel &&
+	    chan_is_connected(window->current_channel, window->server) &&
 	    get_channel_oper(window->current_channel, window->server) &&
 	    (text = get_string_var(STATUS_CHANOP_VAR)))
 		malloc_strcpy(&ptr, text);
@@ -925,10 +1024,11 @@ status_channel(window)
 	Window	*window;
 {
 	int	num;
-	char	*ptr,
+	char	*s, *ptr,
 		channel[IRCD_BUFFER_SIZE + 1];
 
-	if (window->current_channel)
+	s = window->current_channel;
+	if (s && chan_is_connected(s, window->server))
 	{
 		if (get_int_var(HIDE_PRIVATE_CHANNELS_VAR) &&
 		    is_channel_mode(window->current_channel,
@@ -939,7 +1039,7 @@ status_channel(window)
 			ptr = window->current_channel;
 		strmcpy(channel, ptr, IRCD_BUFFER_SIZE);
 		if ((num = get_int_var(CHANNEL_NAME_WIDTH_VAR)) &&
-		    (strlen(channel) > num))
+		    ((int) strlen(channel) > num))
 			channel[num] = (char) 0;
 		/* num = atoi(channel); */
 		ptr = (char *) 0;
@@ -1016,7 +1116,9 @@ status_away(window)
 	char	*ptr = (char *) 0,
 	*text;
 
-	if ((connected_to_server == 1) && !get_int_var(SHOW_STATUS_ALL_VAR)
+	if (connected_to_server == 0)
+		malloc_strcpy(&ptr, empty_string);
+	else if ((connected_to_server == 1) && !get_int_var(SHOW_STATUS_ALL_VAR)
 	    && (window->screen->current_window != window))
 		malloc_strcpy(&ptr, empty_string);
 	else
@@ -1115,7 +1217,9 @@ status_oper(window)
 	char	*ptr = (char *) 0,
 	*text;
 
-	if (get_server_operator(window->server) &&
+	if (!connected_to_server)
+		malloc_strcpy(&ptr, empty_string);
+	else if (get_server_operator(window->server) &&
 			(text = get_string_var(STATUS_OPER_VAR)) &&
 			(get_int_var(SHOW_STATUS_ALL_VAR) ||
 			connected_to_server != 1 || 
@@ -1232,5 +1336,5 @@ status_make_printable(str, n)
 			buffer[pos++] = *s;
 	}
 	buffer[pos] = '\0';
-	strncpy(str, buffer, pos);
+	strncpy((char *) str, buffer, pos);
 }
